;============================================================================
; DB.ASM
;
; Database management module.
;
; The database module is concerned with files and paths whereas the Time and
; Sidebar modules consider files as 'Items'.
;============================================================================




.nolist

include		..\tui.inc
include		..\comn\comn.inc
include		..\sbar\sbar.inc
include		..\time\time.inc

.listall

include		db.inc




; Private functions.

; Here.

LoadGrpSfx	proto
NDBDlgHdlr	proto	hWnd:dword,uMsg:dword,wParam:dword,lParam:dword
BuildArrays	proto
SortFileArray	proto	pArray:dword,Count:dword
InStrg		proto	nOfs:dword,pSrc:dword,nSrc:dword,pPat:dword,nPat:dword
InStrgI		proto	nOfs:dword,pSrc:dword,nSrc:dword,pPat:dword,nPat:dword

; dbupd.asm

UpdateDB	proto	tParam:dword
CountPath	proto	pszPath:dword
ScanPath	proto	pszPath:dword
AddPath		proto	pszPath:dword
AddFile		proto	pwfd:dword




; Definition.


; Database file is read-only,system,hidden.

DBFAttr		equ	FILE_ATTRIBUTE_READONLY or FILE_ATTRIBUTE_SYSTEM or \
			FILE_ATTRIBUTE_HIDDEN




; Data structures


; Database File record structure
; ------------------------------

FREC		struct
  RSpec		dd	0		; Record Specification
  FSpec		dd	0		; File Specification
  FSfx		dd	0		; File Suffix
  TS		dd	0		; File TimeStamp
  Nam		dd	0		; File Name
FREC		ends

; Record Specification:
; b31		0 for File record
; b30		unused
; b29..10	Path Index, 0..1048575
; b09..00	Record length, bytes, 0..1023

RecFileFlag	equ	00000000h
RecPNdxMask	equ	3FFFFC00h	; common to Path record
RecPNdxShft	equ	10		; 10 bits shift
RecLenMask	equ	000003FFh	; common to Path record

; File Specification:
; b31		set if file is searchable
; b30		set if file is in a recycle bin directory
; b29..16	unused
; b15..08	Thumbnail Handler ID, 0 if no handler
; b07..00	Group ID (NB 1 based)

RecSrchFlag	equ	80000000h	; Searchable flag
RecRecyFlag	equ	40000000h	; Recycle bin flag
RecThumMask	equ	0000FF00h	; Thumnail ID
RecThumShft	equ	8		; 8 bits shift
RecGrpMask	equ	000000FFh	; Group ID

; File Suffix:
; File suffix string in uppercase (NB not asciiz), padded to
; the right with zeros if < 4 char.

; TimeStamp:
; Time file was last modified.
; b31..b26	6 bits	1980..2044
; b25..b22	4 bits	month 0..11
; b21..b17	5 bits	day 0..30
; b16..b12	5 bits	hour 0..23
; b11..b06	6 bits	minutes 0..59
; b05..b00	6 bits	seconds 0..59

; File Name:
; Asciiz variable length file name, padded to dword boundary.


; Database Path record structure
; ------------------------------

PREC		struct
  RSpec		dd	0		; Record specification
  Path		dd	0		; Path
PREC		ends

; Record specification:
; b31		1 for Path record
; b30		unused
; b29..10	Path Index, 0..1048575
; b09..00	Record length, bytes, 0..1023

RecPathFlag	equ	80000000h

; Path:
; Asciiz with trailing '\', padded to dword boundary.


; Internal array formats
; ----------------------
;
; File Array	dword	Pointer to File record in database memory image
;
;		The array is sorted on file timestamps in descending order
;		and then the timestamps are removed as no longer required.
;
; Search Array	as for File Array
;
; Path Array	dword	Pointer to record Path string in db memory image
;
;		The Path array is ordered in Path Index order, 0 up.

ARecSize	equ	4		; bytes
ARecMult	equ	2		; 4 = 2^2




.data		; Private to DB module.




szDBF		db	'tui.dbb',0	; Database file name
		align	dword

nPaths		dd	0		; number of Paths in Database
nFiles		dd	0		; number of Files in Database
DBSize		dd	0		; Database size, bytes
pDataBase	dd	0		; ptr, Database start
pPathArray	dd	0		; ptr, Path array start
pFileArray	dd	0		; ptr, File array start
pSrchArray	dd	0		; ptr, Search array start




; UDB thread variables that are shared with rest of DB module.

szUDBEvent	db	"UDBEvent",0	; Event object name
		align	dword
szUDBMutex	db	"UDBMutex",0	; Mutex name
		align	dword
UDBThreadId	dd	0		; UDB thread ID
hUDBThread	dd	0		; handle, UDB Thread
bUDBExit	dd	0		; flag, exit thread

; Store of last search parameters for UpdateDB to redo the search
; on a new database.

LS_Text		db	128 dup (0)	; search text
LS_LenText	dd	0		; length of search text
LS_Flags	dd	0		; search flags
LS_FileCnt	dd	0		; # files found in last search

; Temporary file name.

szTmpFile	db	"tui.tmp",0
		align	dword

; New Database Dialog.

ProgBarTimer	equ	1		; Progress Bar update timer ID
PrgPos		dd	0		; Current position of Bar
bNDBStop	dd	0		; flag, stop Progress Bar update
nResult		dd	0		; flag, exit from NDB dialog
hWaitCursor	dd	0		; handle, wait cursor




; File Suffix data.


; General.

NumSfx		dd	0		; number of suffixes
pSfxList	dd	0		; pointer, Suffix List
nSfxList	dd	0		; bytes allocated to Suffix List

; SFX structure, keeps a suffix with its File specification.

SFX		struc
  Sfx		dd	0		; suffix
  FSpec		dd	0		; File specification
SFX		ends

SFX_MULT	equ	3		; * sizeof SFX

; Default list.
;
; This is the default list of known file suffixes with the initial
; state of their File specification. It is loaded if a Tui initialisation
; file is not found or the suffix section is not present.
;
; The suffix strings are back to front so they are stored in big-endian
; order; eg "TXT." = 54 58 54 2E and ".C" = 00 00 43 2E which become
; numerically 0x02E545854 and 0x02E430000 if loaded into a dword register.
; This makes loading, storing and comparisons easier, at the expense of
; backward strings below.
;
; Note the database, ini file and plugins store suffixes in string form
; or little-endian order.

DefSfxList	label	dword
		; Unknown.
		;
		; Words.
		SFX	<"TXT.",FG_Wrd + (FTH_Txt shl 8) + RecSrchFlag>
		SFX	<"COD.",FG_Wrd>
		SFX	<"FDP.",FG_Wrd>
		; Numbers.
		SFX	<"SLX.",FG_Num + RecSrchFlag>
		; Pictures (GDI+ known read formats).
		SFX	<"PMB.",FG_Pix + (FTH_Img shl 8)>
		SFX	<"FME.",FG_Pix + (FTH_Img shl 8)>
		SFX	<"FIXE",FG_Pix + (FTH_Img shl 8)>
		SFX	<"FIG.",FG_Pix + (FTH_Img shl 8)>
		SFX	<"OCI.",FG_Pix + (FTH_Img shl 8)>
		SFX	<"EPJ.",FG_Pix + (FTH_Img shl 8)>
		SFX	<"GEPJ",FG_Pix + (FTH_Img shl 8)>
		SFX	<"GPJ.",FG_Pix + (FTH_Img shl 8)>
		SFX	<"GNP.",FG_Pix + (FTH_Img shl 8)>
		SFX	<"FIT.",FG_Pix + (FTH_Img shl 8)>
		SFX	<"FFIT",FG_Pix + (FTH_Img shl 8)>
		SFX	<"FMW.",FG_Pix + (FTH_Img shl 8)>
		; Sounds.
		SFX	<"3PM.",FG_Snd>
		SFX	<"VAW.",FG_Snd>
		; Movies.
		SFX	<"IVA.",FG_Vid>
		SFX	<"GPM.",FG_Vid>
		; Net.
		SFX	<"MTH.",FG_Net + RecSrchFlag>
		SFX	<"LMTH",FG_Net + RecSrchFlag>
		SFX	<"LRU.",FG_Net + RecSrchFlag>
		; Help.
		SFX	<"MHC.",FG_Hlp>
		SFX	<"PLH.",FG_Hlp>
		; Archives.
		SFX	<"BAC.",FG_Ark>
		SFX	<"RAR.",FG_Ark>
		SFX	<"PIZ.",FG_Ark>
		; Tools.
		SFX	<"TAB.",FG_Exe>
		SFX	<"MOC.",FG_Exe>
		SFX	<"EXE.",FG_Exe>
		SFX	<"FIP.",FG_Exe>
		; System.
		SFX	<"INI.",FG_Sys + (FTH_Txt shl 8) + RecSrchFlag>
		SFX	<"TAD.",FG_Sys>
		SFX	<"LLD.",FG_Sys>
		SFX	<"XCO.",FG_Sys>
		SFX	<"SYS.",FG_Sys>
		SFX	<"DIG.",FG_Sys>
		SFX	<"JBO.",FG_Sys>
		; Sources.
		SFX	<"MSA.",FG_Src + (FTH_Txt shl 8) + RecSrchFlag>
		SFX	<"CNI.",FG_Src + (FTH_Txt shl 8) + RecSrchFlag>
		SFX	<  "H.",FG_Src + (FTH_Txt shl 8) + RecSrchFlag>
		SFX	<  "C.",FG_Src + (FTH_Txt shl 8) + RecSrchFlag>
		SFX	< "CR.",FG_Src + (FTH_Txt shl 8) + RecSrchFlag>
		SFX	<"PPC.",FG_Src + (FTH_Txt shl 8) + RecSrchFlag>
		; Shortcuts.
		SFX	<"KNL.",FG_Lnk>

NumDefSfx	equ	($ - DefSfxList) / sizeof SFX
		align	dword





; Thumbnail handler Plugins.

NumPlg		dd	0		; Built-ins + Plugins loaded
pPlgList	dd	0		; pointer, Plugin List
nPlgList	dd	0		; bytes allocated to Plugin List


; Plugin specification.

PLG		struct
  hPLG		dd	0		; handle, plugin module
  pPLG		dd	0		; pointer, plugin specification
PLG		ends

PLG_MULT	equ	3		; * sizeof PLG


; Built-in Thumbnail Handler specifications.
;
; These are effectively just placeholders tho the name is used by DB_Export.
; The pointer must however be zero for DB_DeInit.

FTH_BU_Nul	PLGSpec	<0,,,"None">
		align	dword
FTH_BU_Txt	PLGSpec	<0,,,"Text">
		align	dword
FTH_BU_Img	PLGSpec	<0,,,"Image">
		align	dword

NumBUPlg	equ	3


; Finding Plugins.

szPLGPath	db	".\plugin\*.dll",0
ofsPLGDir	equ	9		; length of ".\plugin\"

; Tui functions available to plugins.

TuiFuncs	dd	offset	File_Map
		dd	offset	File_UnMap
		dd	offset	InStrgI




; File Groups.

GRP		struc
  hIcon		dd	0		; handle, Group Icon
  szName	db	8 dup (0)	; includes terminating null
GRP		ends

; Group list.
;
; NB this list is indexed with GroupID-1 (see tui.inc) and there
; must be kGroups members.
;
; Note each record is 4 + (7 char + 0) = 12 bytes.

GrpList		label	dword
		GRP	<0,"Other"  >
		GRP	<0,"Words"  >
		GRP	<0,"Numbers">
		GRP	<0,"Sounds" >
		GRP	<0,"Movies" >
		GRP	<0,"Net"    >
		GRP	<0,"Help"   >
		GRP	<0,"Archive">
		GRP	<0,"Tools"  >
		GRP	<0,"System" >
		GRP	<0,"Sources">
		GRP	<0,"Images" >
		GRP	<0,"Shrtcut">
		GRP	<0,"Recycle">

; Group mask.
;
; A Group is selected if the bit at Group ID-1 is set.

GrpMask		dd	0
DefGrpMask	dd	03FFFh		; all Groups visible




; Database initialisation.

szDBIS_Grp	db	"Group",0	; Section
		align	dword
szDBIK_GrpMsk	db	"Mask",0	; Key
		align	dword

szDBIS_Sfx	db	"Suffix",0	; Section
		align	dword
szDBIK_SfxNum	db	"Sfxs",0	; Key
		align	dword




; Export Database.

EXszFileName	db	"find.txt",0
		align	dword
EXszPaths	db	"Total Paths: ",0
		align	dword
EXszFiles	db	"Total Files: ",0
		align	dword
EXszSrch	db	"Last Search: ",0
		align	dword
EXszSrchFiles	db	"Files Found: ",0
		align	dword
EXszGroups	db	"Used Groups: ",0
		align	dword
EXszHeading	db	"Timestamp",TB,"   Text,Thumb",TB,"Group"
		db	TB," Sfx",TB,"File Name",TB,"File Path",0
		align	dword
EXszFlagSet	db	"   1,",0
		align	dword
EXszFlagClr	db	"   0,",0
		align	dword




; Case translation table for InStrgI.

align	16
CaseTable	label	byte
	db	  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15
	db	 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31
	db	 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47
	db	 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63
	db	 64, 97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,111
	db	112,113,114,115,116,117,118,119,120,121,122, 91, 92, 93, 94, 95
	db	 96, 97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,111
	db	112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127
	db	128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143
	db	144,145,146,147,148,149,150,151,152,153,154,155,156,156,158,159
	db	160,161,162,163,164,165,166,167,168,169,170,171,172,173,173,175
	db	176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191
	db	192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207
	db	208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223
	db	224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239
	db	240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255




; Object allocation.

DB_OAF		dd	0

; Flags, bit set indicates object/resource etc are allocated.
; These are used in the deallocation routine.

DB_OAF_DBOpen	equ	1 shl 0		; DB file open




; Database module error handling.


; Database module error codes.

ERR_DB_Module	equ	1100h

ERR_DBInit	equ	ERR_DB_Module + 0	; (placeholder only)
ERR_EventObj	equ	ERR_DB_Module + 1
ERR_UpdMutex	equ	ERR_DB_Module + 2
ERR_UpdThread	equ	ERR_DB_Module + 3

; DB_Init error messages.
;
; sERR_DBInit has one of the following error messages appended.

sERR_DBInit	db	"Database Init failed - ",0
sERR_EventObj	db	"Event Object creation",0
sERR_UpdMutex	db	"Update Mutex creation",0
sERR_UpdThread	db	"Update Thread creation",0
		align	dword

ErrorMsgs	dd	offset sERR_DBInit
		dd	offset sERR_EventObj
		dd	offset sERR_UpdMutex
		dd	offset sERR_UpdThread

kErrors		equ	4




.code




; Public functions.

include		dbpub.asm




; Private functions.

option		proc:private




include		dbupd.asm




;----------------------------------------------------------------------------
LoadGrpSfx	proc	uses ebx esi edi
;---------
;
; Load Group and Suffix data.
;
; Auxiliary routine for DB_Init.
;----------------------------------------------------------------------------


	local	SfxKey[4]	:byte		; sfx key names, 000->999
	local	IniBuf[32]	:byte		; suffix keys
	local	HaveIni		:dword		; flag, ini file + section ok
	local	wfd		:WIN32_FIND_DATA
	local	hFind		:dword		; handle, find
	local	szPLGLib[64]	:byte		; path of plugin dll
	local	Sfx2		:dword		; possible second suffix


; Load our Group icons from tui.exe embedded resources and insert their
; handles into the Group List.


	mov	edi, offset GrpList
	xor	ebx, ebx
	inc	ebx

	.repeat

		invoke	LoadIconA, G_hTui, ebx

		; If we get an error here then eax = 0 and no icon will be
		; shown for this Group's Items in the Time Window.

		mov	[edi + GRP.hIcon], eax
		add	edi, sizeof GRP
		inc	ebx

	.until	(ebx > kGroups)


; Load selected Groups and known suffixes.


	; Query ini file for number of suffixes it has.
	; If no ini file or no suffix section/key then load defaults.

	xor	eax, eax
	mov	HaveIni, eax			; preset flag

	invoke	File_TuiQryIni

	.if	(!CARRY?)			; ini found

		; Get Group mask.

		invoke	File_RdIniKey, addr szDBIS_Grp, addr szDBIK_GrpMsk,
				addr IniBuf, sizeof IniBuf

		.if	(eax)

			invoke	htodw, addr IniBuf

			; Jic ini file has a null value.

			.if	(eax == 0)
				mov	eax, DefGrpMask
			.endif

			mov	GrpMask, eax

		.else

			mov	eax, DefGrpMask
			mov	GrpMask, eax

		.endif

		; Get number of suffixes.

		invoke	File_RdIniKey, addr szDBIS_Sfx, addr szDBIK_SfxNum,
				addr IniBuf, sizeof IniBuf

		.if	(eax)
			invoke	atodw, addr IniBuf
			mov	HaveIni, 1
		.else				; error
			mov	eax, NumDefSfx
		.endif

	.else

		mov	eax, DefGrpMask
		mov	GrpMask, eax
		mov	eax, NumDefSfx

	.endif

	mov	NumSfx, eax


	; Allocate memory for the Suffix List.
	;
	; Add 8 records to allow for initial additions.
	; (it will be expanded again if required by DB_SetTypeSpec)

	shl	eax, SFX_MULT			; #Sfx * sizeof SFX
	add	eax, sizeof SFX * 8
	mov	nSfxList, eax
	invoke	Mem_Alloc, eax
	mov	pSfxList, eax


	; And load, either from ini file or defaults.

	.if	(HaveIni)

		mov	edi, pSfxList
		xor	ecx, ecx

		.repeat

			push	ecx

			; Make Key name and read.

			invoke	dwtoa, ecx, addr SfxKey
			invoke	File_RdIniKey, addr szDBIS_Sfx,	addr SfxKey,
				addr IniBuf, sizeof IniBuf

			; Parse and store to Suffix List.

			xor	ebx, ebx
			mov	ah, ','

			.repeat

				mov	al, IniBuf[ebx]
				.break	.if (al == ah)
				inc	ebx

			.until	(ebx == sizeof IniBuf)

			.if	(ebx == sizeof IniBuf)
				jmp	@@BadEntry
			.endif

			; Load the File spec first.

			inc	ebx

			invoke	htodw, addr IniBuf[ebx]
			add	edi, SFX.FSpec
			stosd

			; Now the suffix.
			;
			; Clear the next 4 char in the buffer, including
			; the comma, so they don't appear in the suffix.

			dec	ebx
			xor	eax, eax
			mov	dword ptr IniBuf[ebx], eax
			mov	eax, dword ptr IniBuf
			mov	[edi - sizeof SFX], eax

			@@BadEntry:

			pop	ecx
			inc	ecx

		.until	(ecx == NumSfx)

	.else

		; Load the default list.

		mov	esi, offset DefSfxList
		mov	edi, pSfxList
		mov	ecx, NumSfx
		shl	ecx, SFX_MULT/2		; dwords in list
		rep	movsd

	.endif


; Scan for Thumbnail plugins and load them.


	; Allocate memory for the Plugin List.
	;
	; Create 8 records as a default.

	mov	eax, sizeof PLG * 8
	mov	nPlgList, eax
	invoke	Mem_Alloc, eax
	mov	pPlgList, eax
	mov	edi, eax


	; Load our built-in plugins to the first 3 entries.

	xor	edx, edx			; handles are zero

	mov	[edi], edx
	mov	eax, offset FTH_BU_Nul
	mov	[edi + 4], eax

	mov	[edi + 8], edx
	mov	eax, offset FTH_BU_Txt
	mov	[edi + 12], eax

	mov	[edi + 16], edx
	mov	eax, offset FTH_BU_Img
	mov	[edi + 20], eax

	add	edi, sizeof PLG * NumBUPlg

	mov	NumPlg, NumBUPlg


	; Scan plugins directory.

	invoke	szCopy, addr szPLGPath, addr szPLGLib

	invoke	FindFirstFileA, addr szPLGPath, addr wfd
	mov	hFind, eax

	mov	ebx, eax

	.while	(ebx > 0)

		; Check we have enough room.

		mov	eax, NumPlg
		shl	eax, PLG_MULT

		.if	(eax == nPlgList)

			; No, add another 8 records.

			push eax			; save current size

			add	eax, sizeof PLG * 8
			mov	nPlgList, eax
			invoke	Mem_ReAlloc, addr pPlgList, eax

			; Reset our pointer to the end of the array.

			pop	eax

			mov	edi, pPlgList
			add	edi, eax

		.endif

		; Found a plugin, attempt to load it.

		mov	szPLGLib[ofsPLGDir], 0	; (terminator)
		lea	esi, wfd.cFileName
		invoke	szCatStr, addr szPLGLib, esi
		invoke	LoadLibrary, addr szPLGLib

		.if	(eax)			; success

			; There is only one function exported so
			; it will be at ordinal 1.

			mov	[edi + PLG.hPLG], eax
			invoke	GetProcAddress, eax, 1

			.if	(eax)		; success

				; Call PlugSetup. The return value is
				; a pointer to the Plugin's internal
				; specification structure.

				push	offset TuiFuncs
				call	eax
				mov	[edi + PLG.pPLG], eax
				add	edi, sizeof PLG
				inc	NumPlg

			.endif

		.endif

		invoke	FindNextFileA, hFind, addr wfd
		mov	ebx, eax

	.endw

	invoke	FindClose, hFind


; Now update the Suffix list with the found Thumbnail handlers.


	mov	ebx, NumBUPlg		; first possible external plugin

	.while	(ebx < NumPlg)

		; Get Handler suffix(s) and scan Suffix List for it/them.

		invoke	DB_GetThumData, ebx, T_GETSFX1
		mov	edi, eax

		invoke	DB_GetThumData, ebx, T_GETSFX2
		mov	Sfx2, eax

		mov	esi, pSfxList
		mov	ecx, NumSfx

		.repeat

			mov	eax, Sfx2
			mov	edx, [esi + SFX.Sfx]

			.if	((edi == edx) || (eax == edx))

				; Found a suffix, load handler id.

				mov	eax, ebx
				shl	eax, RecThumShft
				or	[esi + SFX.FSpec], eax

			.endif

			add	esi, sizeof SFX
			dec	ecx

		.until	(ZERO?)

		inc	ebx

	.endw


; All done, exit.


	ret


LoadGrpSfx	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
NDBDlgHdlr	proc	uses ebx esi edi,
			hWnd:dword,uMsg:dword,wParam:dword,lParam :dword
;---------
;
; Create New Database Dialog message handler.
;
;
; Exit		!0	Message processed
;
;		0	Message not processed - the dialog manager performs
;			the default dialog operation for the message.
;
;		End:
;
;		nResult		1	Finished creating database
;				0	Tui exit requested by user
;
;
; External	UDBStatus bit UDB1stDone is set when UpdateDB has finished
;		the first database update after Tui is loaded.
;----------------------------------------------------------------------------


	mov	eax, uMsg

	cmp	eax, WM_INITDIALOG
	jz	@@OnInit
	cmp	eax, WM_TIMER
	jz	@@OnTimer
	cmp	eax, WM_COMMAND
	jz	@@OnCommand
	cmp	eax, WM_CLOSE
	jz	@@OnClose

	xor	eax, eax			; Msg unhandled
	jmp	@@ExitHandler


@@OnInit:	;------------------------------------------------------------
		; Initialise the Dialog box.


	; Set the Progress bar range and color.

	invoke	SendDlgItemMessage,hWnd,ID_ND_PGB1,PBM_SETRANGE32,0,100
	invoke	SendDlgItemMessage,hWnd,ID_ND_PGB1,PBM_SETBARCOLOR,0,CBlue

	; We may need the wait cursor if a cancel on a drive with many files.

	invoke	LoadCursorA, NULL, IDC_WAIT
	mov	hWaitCursor, eax

	jmp	@@MsgHandled


@@OnTimer:	;------------------------------------------------------------
		; Progress Bar Timer.


	; If Progress Bar disabled then ignore this message.
	; This will only come from the Fatal ErrorHandler.

	.if	(bNDBStop)
		jmp	@@MsgHandled
	.endif

	; Ensure it is our timer.

	.if	(wParam != ProgBarTimer)
		jmp	@@MsgHandled
	.endif


	.if	!(G_UDBStatus & UDB1stDone)

		; Still creating, update progress bar position.

		invoke	SendDlgItemMessage, hWnd, ID_ND_PGB1, PBM_SETPOS,
				PrgPos, 0

		inc	PrgPos

		.if	(PrgPos > 100)
			mov	PrgPos, 0
		.endif

	.else

		; UpdateDB has finished creating the database, kill timer
		; and exit dialog.

		invoke	KillTimer, hWnd, ProgBarTimer

		invoke	SuspendThread, hUDBThread

		mov	nResult, 1

		invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL

	.endif

	jmp	@@MsgHandled


@@OnCommand:	;------------------------------------------------------------
		; Mouse click on buttons/controls.


	movzx	eax, wPLo

	.if	(eax == IDOK)


		; Set the UpdateDB thread working at an increased priority
		; and create the database. We must trigger the Event wait
		; in UpdateDB for this first entry.

		invoke	SetEvent, G_UDBhEvent

		invoke	SetThreadPriority, hUDBThread, THREAD_PRIORITY_HIGHEST
		invoke	ResumeThread, hUDBThread

		; Start our progress bar timer.

		invoke	SetTimer, hWnd, ProgBarTimer, 100, NULL


	.elseif	(eax == IDCANCEL)


		.if	(G_UDBStatus)

			; We have started creating the database but have not
			; finished, send abort and wait for thread to exit.
			;
			; (UDBStatus <= !0 as soon as UpdateDB is entered)
			;
			; We can wait here up to the time taken to scan
			; a drive partition, which may be a while if it
			; has many files. Let's assume 2 mins max and then
			; just terminate it.

			invoke	SetCursor, hWaitCursor	; Show wait cursor

			mov	bUDBExit, 1

			invoke	WaitForSingleObject, hUDBThread, 120000

			.if	(eax != WAIT_OBJECT_0)
				jmp	@@Terminate
			.endif


		.else

			@@Terminate:

			; We have not started so just terminate the
			; thread as it is still suspended.

			invoke	TerminateThread, hUDBThread, NULL

		.endif

		; Release thread handle now and clear it to inform
		; DB_DeInit not to wait - which will never be signalled.

		invoke	CloseHandle, hUDBThread
		xor	eax, eax
		mov	hUDBThread, eax

		mov	nResult, 0

		invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL

	.endif

	jmp	@@MsgHandled


@@OnClose:	;------------------------------------------------------------
		; Close Dialog box, Alt-F4 etc.


	invoke	EndDialog, hWnd, nResult


@@MsgHandled:	;------------------------------------------------------------
		; Exit, Dialog message handled.


	mov	eax, TRUE


@@ExitHandler:


	ret


NDBDlgHdlr	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
BuildArrays	proc	uses ebx esi edi
;----------
;
; Build Path, File and Search arrays.
;
; NB First called from DB_Init and thereafter only from UpdateDB.
;
;
; Entry		External:
;
;		pDatabase	pointer to loaded Database
;
;
; Exit		External:
;
;		nPaths		number of Path records in Database
;		nFiles		number of File records in Database
;		pPathArray	pointer to start of Path array
;		pFileArray	pointer to start of File array
;		pSrchArray	pointer to start of Search array
;
;
; Note
;
; As we must keep a TimeStamp together with its file record pointer for the
; sort, we need 2 dwords per file entry or (Files*8) bytes of memory.
; After the sort the TimeStamp is redundant so the final version of the File
; array has 1 dword per entry with the Search array of equivalent size - so
; we can fit them both in the initial memory allocation.
;
; It occurs that if a nested search capability is required then dimensioning
; the array to Files*16 and using the upper half as the sort buffer would
; leave 2x the Search array free to provide 3 levels of searching and there
; would be no need for the sort routine to malloc/dalloc.
;----------------------------------------------------------------------------


	local	DummyFrame	:dword		; (for call stack tracing)


; Release prior array allocation if allocated.


	invoke	Mem_DeAlloc, addr pPathArray
	invoke	Mem_DeAlloc, addr pFileArray


; Count Paths and Files.


	xor	eax, eax
	mov	nPaths, eax
	mov	nFiles, eax
	mov	edi, pDataBase

	@@CountPF:

	mov	eax, [edi]
	or	eax, eax			; If spec is 0
	jz	@@EndCountPF			;  then end of DB.

	mov	edx, eax
	and	eax, RecLenMask			; mask in length
	test	edx, RecPathFlag		; Record type
	jnz	@@CntPathRec

	inc	nFiles
	add	edi, eax
	jmp	@@CountPF

	@@CntPathRec:

	inc	nPaths
	add	edi, eax
	jmp	@@CountPF

	@@EndCountPF:


; Allocate array memory.


	mov	eax, nFiles
	shl	eax, ARecMult			; * array record size
	shl	eax, 1				; double it
	invoke	Mem_Alloc, eax
	mov	pFileArray, eax

	mov	eax, nPaths
	shl	eax, ARecMult			; * array record size
	invoke	Mem_Alloc, eax
	mov	pPathArray, eax


; Load Path and File arrays.
;
; The Path array is ordered in Path Index order, 0 up.


	mov	esi, pDataBase
	mov	edi, pFileArray
	mov	ebx, pPathArray

	.while	TRUE

		mov	ecx, [esi]		; If record spec is 0,
		.break	.if (ecx == 0)		;  then end of Database.

		.if	!(ecx & RecPathFlag)

			; File record, store Timestamp and record pointer.

			mov	eax, [esi + FREC.TS]
			mov	[edi], eax
			add	edi, ARecSize
			mov	[edi], esi
			add	edi, ARecSize

		.else

			; Path record, store pointer to Path string.

			mov	eax, esi
			add	eax, PREC.Path
			mov	[ebx], eax
			add	ebx, ARecSize

		.endif

		and	ecx, RecLenMask		; mask in length
		add	esi, ecx

	.endw


; Sort the File array.


	invoke	SortFileArray, pFileArray, nFiles


; Condense the File Array to its final form and set the Search
; Array pointer.


	mov	edi, pFileArray
	mov	esi, edi
	add	esi, ARecSize			; first pointer
	xor	ecx, ecx

	.repeat

		; Move pointer down.

		ptrOffset equ	ARecSize*2

		mov	eax, [esi + ecx*ptrOffset]
		mov	[edi + ecx*ARecSize], eax
		inc	ecx

	.until	(ecx == nFiles)

	mov	eax, ecx			; (nFiles)
	shl	eax, ARecMult
	mov	edx, eax
	add	eax, pFileArray
	mov	pSrchArray, eax


	ret


BuildArrays	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
SortFileArray	proc	uses ebx esi edi,pArray:dword,Count:dword
;------------
;
; Radix sort File array on TimeStamp.
;
; Only called from BuildArrays.
;
;
; Entry		pArray		ptr, start of File array
;		Count		number of items to sort
;
;
; Exit		Array		sorted
;
;
; A radix is a position in a number. In the decimal system a radix is just a
; digit in a decimal number. For example the number 42 has two digits, or two
; radices, which are 4 and 2. Our radix is 8 bits wide (base 256). Eg the
; hex number 0x1ADB has two radices, 1A and DB. The Radix Sort gets its name
; from these radices, because the method first sorts the input values
; according to their first radix, then according to the second one, and so on.
; It is thus a multipass sort, and the number of passes equals the number of
; radices in the input values. For 32 bit integers 4 passes are required since
; we have four byte radices. This is why the Radix Sort is often called Byte
; Sort. Note the sort passes start with the least significant radix.
;
; The Radix sort is 'stable' which means two equal values in the input will be
; in the same order in the output. Thus each pass required to sort mulit-radix
; items does not destroy the order of equivalent items in the previous pass.
;
; The number of operations required to sort an input array of N items, each
; having M radices, is N*M operations. To this is added the building of the
; index table. If the number of items in the array is small then the time
; taken to create the index table dominates the sort time.
;
; Optimisations:
;
; Create all four histograms in 1 pass.
;
;	pros:	read input array once instead of four times
;		faster for approx. > 50000 items
;
;	cons:	Histogram is 4Kb instead of 1Kb
;		slower for approx. < 50000 items
;
;
; Array notes:
;
; The input array has 8 byte records with the first 4 containing the file
; TimeStamp, on which the sort is done. Thus the number of members in the
; array, from the point of view of sorting, is the number of Timestamps.
; So below we bump array pointers by 8 bytes to skip to the next Timestamp.
;
; The sort is in descending order as we want to sort latest Timestamps
; toward earliest.
;----------------------------------------------------------------------------


; Define our array record size and shift factor.


	RSize	equ		8		; bytes
	RMult	equ		3		; *8

	local	index[256]	:dword		; index array
	local	radix		:dword		; numbered 0 up
	local	hMem		:dword		; handle, buffer array
	local	pBuf		:dword		; ptr, buffer array


; Allocate a buffer equivalent to the input array.


	mov	eax, Count
	shl	eax, RMult			; * array record size
	invoke	Mem_Alloc, eax
	mov	hMem, eax
	mov	pBuf, eax


; We do a pass for each radix.
;
; Pass 1	sort from input array to buffer on ls byte
; Pass 2	sort from buffer to input array on byte 2
; Pass 3	sort from input array to buffer on byte 3
; Pass 4	sort from buffer to input array on ms byte
;
; Thus leaving our final sorted result in the input array.


	xor	eax, eax
	mov	radix, eax


@@NextRadix:


	; 1. Build the index table.
	;
	; The index table is a 256 entry table of offsets into a destination.
	; It tells us, for each possible radix input byte, where we should
	; store the corresponding input item into the destination array.
	;
	; It is built in two passes, one to compute the distribution of bytes
	; in the input flow (histogram) and another one to create the index
	; table according to this distribution.
	;
	; For example, if we have an input array of 12 members containing
	; decimals in the range 0..9. This will have an index array[10], 0..9
	;
	; Input[0] [1] [2] [3] [4] [5] [6] [7] [8] [9] [10][11]
	;       3   0   5   6   9   0   3   8   8   2   4   3

	; Reset index table.

	mov	ecx, 256
	lea	edi, index
	xor	eax, eax
	rep	stosd

	; Count occurrences of each byte value for this radix, in the
	; input array, to create a histogram or distribution history.
	;
	; R denotes radix value.
	;
	; for i = 0 to N-1
	;   index[sourceR[i]] = index[sourceR[i]] + 1
	; rof
	;
	; Our example will form the histogram:
	;
	; index[0] [1] [2] [3] [4] [5] [6] [7] [8] [9]
	;       2   0   1   3   1   1   1   0   2   1

	mov	ecx, Count
	mov	ebx, pArray
	add	ebx, radix		; ebx = source radix value
	lea	esi, index		; esi = index table
					; eax = 0 from above
	@@CountValue:
	mov	al, [ebx]
	inc	dword ptr [esi + eax*4]
	add	ebx, RSize
	dec	ecx
	jnz	@@CountValue

	; Transform the counts into offsets by summing them.
	;
	; for i = 1 to 255
	;   index[i] = index[i] + index[i-1]
	; rof
	;
	; Our example index array now has offsets:
	;
	; index[0] [1] [2] [3] [4] [5] [6] [7] [8] [9]
	;       2   2   3   6   7   8   9   9  11  12

	; Descending sort.

	mov	ecx, 254		; esi = index(0)
	lea	edi, [esi + 4]		; edi = index(1)

	@@MakeOffset:
	mov	eax, [edi + ecx*4]
	add	[esi + ecx*4], eax
	dec	ecx
	jns	@@MakeOffset

	if 0
	; Ascending sort, not used here.
	mov	ecx, 1			; esi = index(0)
	lea	edi, [esi - 4]		; edi = index(0)-4

	@@MakeOffset:
	mov	eax, [edi + ecx*4]
	add	[esi + ecx*4], eax
	inc	cl
	jnz	@@MakeOffset
	endif


	; 2. Sort the source array items with respect to the current radix.
	;
	; Source and destination are arrays of N items.
	; R denotes radix value, I denotes array item.
	;
	; for i = N-1 to 0
	;   R = sourceR[i]
	;   index[R] = index[R] - 1
	;   offset = index[R]
	;   destI[offset] = sourceI[i]
	; rof
	;
	; Our example input array is now sorted:
	;
	; index[0] [1] [2] [3] [4] [5] [6] [7] [8] [9]
	;       2   2   3   6   7   8   9   9  11  12
	;
	; Input[0] [1] [2] [3] [4] [5] [6] [7] [8] [9] [10][11]
	;       3   0   5   6   9   0   3   8   8   2   4   3
	;
 	;  Dest[0] [1] [2] [3] [4] [5] [6] [7] [8] [9] [10][11]
	;
	;  11                       3                           ndx[3] <- 5
	;  10                           4                       ndx[4] <- 6
	;   9           2                                       ndx[2] <- 2
	;   8                                           8       ndx[8] <- 10
	;   7                                       8           ndx[8] <- 9
	;   6                   3                               ndx[3] <- 4
	;   5       0                                           ndx[0] <- 1
	;   4                                               9   ndx[9] <- 11
	;   3                                   6               ndx[6] <- 8
	;   2                               5                   ndx[5] <- 7
	;   1   0                                               ndx[0] <- 0
	;   0               3                                   ndx[3] <- 3
	;
 	;   =   0   0   2   3   3   3   4   5   6   8   8   9

	mov	ecx, Count
	dec	ecx
	mov	ebx, ecx
	shl	ebx, RMult		; * record size
	add	ebx, pArray		; ebx = source[N-1]
	mov	esi, ebx
	add	esi, radix		; esi = source[N-1] radix value
	mov	edi, pBuf		; edi = dest[0]

	@@Sort:

	; Retrieve radix value of the source item.

	xor	eax, eax
	mov	al, [esi]

	; Decrement this value's offset and then retrieve it.

	dec	dword ptr [index + eax*4]
	mov	eax, [index + eax*4]

	; Move source item into its required position in the destination.

	mov	edx, [ebx]			; move TimeStamp
	mov	[edi + eax*RSize], edx
	mov	edx, [ebx + 4]			; then record pointer
	mov	[edi + eax*RSize + 4], edx

	sub	esi, RSize			; adjust pointers
	sub	ebx, RSize

	dec	ecx
	jns	@@Sort				; ("jns" as must count 0)


	; 3. Swap array and buffer pointers and do next radix.

	mov	eax, pArray
	mov	ebx, pBuf
	mov	pArray, ebx
	mov	pBuf, eax

	inc	radix
	cmp	radix, 3		; last?
	jbe	@@NextRadix		; no


; Exit, releasing our buffer.


	invoke	Mem_DeAlloc, addr hMem


	ret


SortFileArray	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
InStrg		proc	uses ebx esi edi, nOfs:dword,pSrc:dword,nSrc:dword,
			pPat:dword,nPat:dword
;-----
;
; Case Sensitive search for a substring inside a string.
;
;
; Entry		nOfs		start offset into source, 0 based
;		pSrc		pointer to source string
;		nSrc		length of source string
;		pPat		pointer to pattern string
;		nPat		length of pattern string
;
;
; Exit		carry clear	eax	offset of pattern in source, 0 based
;		carry set	pattern not found or nPat > nSrc
;----------------------------------------------------------------------------


; Setup.


	mov	ecx, nSrc
	mov	eax, nPat
	sub	ecx, eax		; nSrc - nPat

	.if	(CARRY?)
		jmp	@@Exit		; nSrc < nPat
	.endif


; Do a byte scan if only 1 byte in pattern.


	.if	(eax == 1)

		mov	edi, pSrc
		add	edi, nOfs
		mov	ecx, nSrc
		sub	ecx, nOfs
		inc	ecx

		mov	esi, pPat
		mov	al, [esi]

		repne	scasb

		.if	(ecx)
			mov	eax, nSrc
			sub	eax, ecx
			clc
		.else
			stc
		.endif

		jmp	@@Exit

	.endif


; Else we have to jump thru the hoop.


	dec	eax
	mov	nPat, eax		; offset, last char of pattern

	; |<--- Source string (nSrc) ---->|
	;                       |  nPat   |
	; |<-- N char to scan -->|
	;      (ecx = -N)        ^<- source pointer, esi

	mov	esi, pSrc
	inc	ecx
	add	esi, ecx
	neg	ecx
	add	ecx, nOfs		; (add offset)

	mov	edi, pPat
	mov	al, [edi]		; first char of pattern
	jmp	@@ScanLoop


; Search.


	align	dword
@@Restart:

	inc ecx				; next char in source

	; Scan for first byte of pattern in source string.

	@@ScanLoop:

	cmp	al, [esi + ecx]
	je	@@SrchPattern		; found it, now rest of pattern

	inc	ecx			; move up in source
	js	@@ScanLoop

	stc				; End of source, not found.
	jmp	@@Exit

	; Examine for pattern from this point in the source.

	@@SrchPattern:

	lea	ebx, [esi + ecx]	; current source scan address
	mov	edx, nPat		; pattern length - 1

	; Search for pattern backward. This saves time as if the last
	; char doesn't match then we don't have the pattern.

	@@TestMatch:

	mov	ah, [ebx + edx]		; source char
	cmp	ah, [edi + edx]		; compare with pattern char
	jne	@@Restart		; mismatch, examine next source char

	dec	edx
	jnz	@@TestMatch

	; Found match.

	add	ecx, nSrc
	sub	ecx, nPat
	mov	eax, ecx
	clc


@@Exit:


	ret


InStrg		endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
InStrgI		proc	uses ebx esi edi, nOfs:dword,pSrc:dword,nSrc:dword,
			pPat:dword,nPat:dword
;------
;
; Case InSensitive search for a substring inside a string, using a table.
;
;
; Entry		nOfs		start offset into source, 1 based (0 = none)
;		pSrc		pointer to source string
;		nSrc		length of source string
;		pPat		pointer to pattern string
;		nPat		length of pattern string
;
;
; Exit		carry clear	eax	offset of pattern in source, 0 based
;		carry set	pattern not found or nPat > nSrc
;----------------------------------------------------------------------------


	local	p1	:byte		; first pattern case table value
	local	m2	:byte		; subsequent values


; Setup.


	mov	ecx, nSrc
	mov	eax, nPat
	sub	ecx, eax		; nSrc - nPat

	.if	(CARRY?)
		jmp	@@Exit		; nSrc < nPat
	.endif


; Do a byte scan if only 1 byte in pattern.


	.if	(eax == 1)

		mov	edi, pSrc
		add	edi, nOfs
		mov	ecx, nSrc
		sub	ecx, nOfs
		inc	ecx

		mov	esi, pPat
		mov	al, [esi]
		mov	al, CaseTable[eax]
		xor	edx, edx

		@@Scan:
		mov	dl, [edi]
		cmp	al, CaseTable[edx]
		jnz	@@Next

		mov	eax, nSrc
		dec	ecx
		sub	eax, ecx
		clc
		jmp	@@Exit

		@@Next:
		inc	edi
		dec	ecx
		jnz	@@Scan

		stc
		jmp	@@Exit

	.endif


; Else we have to jump thru the hoop.


	dec	eax
	mov	nPat, eax		; offset, last char of pattern

	; |<--- Source string (nSrc) ---->|
	;                       |  nPat   |
	; |<-- N char to scan -->|
	;      (ecx = -N)        ^<- source pointer, esi

	mov	esi, pSrc
	inc	ecx
	add	esi, ecx
	neg	ecx
	add	ecx, nOfs		; (start offset)

	mov	edi, pPat
	xor	eax, eax
	mov	al, [edi]		; first char of pattern
	mov	al, CaseTable[eax]
	mov	p1, al			;  as it's table value.

	jmp	@@ScanLoop


; Search.


	align	dword
@@Restart:

	inc ecx				; next char in source

	; Scan for first byte of pattern in source string.

	@@ScanLoop:

	mov	al, [esi + ecx]
	mov	al, CaseTable[eax]
	cmp	al, p1
	je	@@SrchPattern		; found it, now rest of pattern

	inc	ecx			; move up in source
	js	@@ScanLoop

	stc				; End of source, not found.
	jmp	@@Exit

	; Examine for pattern from this point in the source.

	@@SrchPattern:

	lea	ebx, [esi + ecx]	; current source scan address
	mov	edx, nPat		; pattern length

	; Search for pattern backward. This saves time as if the last
	; char doesn't match then we don't have the pattern.

	@@TestMatch:

	mov	al, [ebx + edx]		; source char
	mov	al, CaseTable[eax]
	mov	m2, al
	mov	al, [edi + edx]		; compare with pattern char
	mov	al, CaseTable[eax]
	cmp	al, m2
	jne	@@Restart		; mismatch, examine next source char

	dec	edx
	jnz	@@TestMatch

	; Found match.

	add	ecx, nSrc
	sub	ecx, nPat
	mov	eax, ecx
	clc


@@Exit:


	ret


InStrgI		endp
;----------------------------------------------------------------------------




end		; DB module.




;============================================================================
