;============================================================================
; SBAR.ASM
;
; Sidebar module.
;
;
; Window focus for SideBar and Time Windows
;
; The primary focus is the Time window which is topmost and full screen to
; allow it to overlay the TaskBar. The SideBar is always available at the
; right edge of the screen and will overlay the Time window when it is
; opened. A requirement is any SideBar or Time window dialogs should not
; disappear behind the Time window as then the user will need to alt+tab
; to see them again - not good.
;
; After much dicking around the easiest method found was to close any dialog
; if the user clicks outside it. This is done by using the WM_ACTIVATE
; message to detect if the dialog is being de-activated and then to close it.
;============================================================================




.nolist

include		..\tui.inc
include		..\comn\comn.inc
include		..\db\db.inc
include		..\time\time.inc

.listall

include		sbar.inc




; Private functions.

SBarWndHdlr	proto	hWnd:dword,uMsg:dword,wParam:dword,lParam:dword
FindDlgHdlr	proto	hWnd:dword,uMsg:dword,wParam:dword,lParam:dword
GrpDlgHdlr	proto	hWnd:dword,uMsg:dword,wParam:dword,lParam:dword
InitGrpLV	proto	hWnd:dword
ExptDlgHdlr	proto	hWnd:dword,uMsg:dword,wParam:dword,lParam:dword
OptnDlgHdlr	proto	hWnd:dword,uMsg:dword,wParam:dword,lParam:dword
SetColor	proto	hWnd:dword,Ndx:dword,CurrColor:dword
CCDlgHdlr	proto	hWnd:dword,uMsg:dword,wParam:dword,lParam:dword
HelpDlgHdlr	proto	hWnd:dword,uMsg:dword,wParam:dword,lParam:dword




; Mouse detection timer ID.

MouseDetTimer	equ	1




.data		; Private to SideBar module.




; SideBar.


szSBarCN	db	"SBar", 0	; Sidebar window class name
		align	dword

hSBarWnd	dd	0		; Sidebar window handle
SBarIsMax	dd	0		; flag, SideBar maximised
SBarXLeft	dd	0		; Sidebar maximised left coord.
hForeGndWnd	dd	0		; handle, ForeGroundWindow
L_hTimeWnd	dd	0		; handle, Time window (local copy)
hTBWnd		dd	0		; handle, ToolBar window


; ToolTips for ToolBar buttons.

szTime		db	"Time",0
		align	dword
szDesk		db	"Desktop",0
		align	dword
szFind		db	"Find",0
		align	dword
szGroup		db	"Groups",0
		align	dword
szThum		db	"Toggle Thumbnails",0
		align	dword
szExpt		db	"Export",0
		align	dword
szOpt		db	"Options",0
		align	dword
szHelp		db	"Help",0
		align	dword
szExit		db	"Exit",0
		align	dword


; ToolBar button specifications.

TBBmpXY		equ	16	; 16x16 pixels

TBButtons	TBBUTTON	<SB_BMP_TIME,ID_SB_BTTIME, \
				TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0>
		TBBUTTON	<SB_BMP_DESK,ID_SB_BTDESK, \
				TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0>
		TBBUTTON	<0,0,0,TBSTYLE_SEP,0,0>
		TBBUTTON	<SB_BMP_FIND,ID_SB_BTFIND, \
				TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0>
		TBBUTTON	<SB_BMP_GRP,ID_SB_BTGRP, \
				TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0>
		TBBUTTON	<SB_BMP_THUM,ID_SB_BTTHUM, \
				TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0>
		TBBUTTON	<0,0,0,TBSTYLE_SEP,0,0>
		TBBUTTON	<SB_BMP_EXPT,ID_SB_BTEXPT, \
				TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0>
		TBBUTTON	<SB_BMP_OPTN,ID_SB_BTOPTN, \
				TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0>
		TBBUTTON	<SB_BMP_HELP,ID_SB_BTHELP, \
				TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0>
		TBBUTTON	<0,0,0,TBSTYLE_SEP,0,0>
		TBBUTTON	<SB_BMP_EXIT,ID_SB_BTEXIT, \
				TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0>
		align	dword




; Dialogs.


; Find Dialog.

bChkCase	dd	0		; flag, Case Sensitive
bChkInName	dd	0		; flag, check in Name
bChkInData	dd	0		; flag, check in Data
bChkInPath	dd	0		; flag, check in Path
bChkNameEnd	dd	0		; flag, check end of Name

SrchTxtLen	dd	0		; search pattern length
SrchText	db	128 dup (0)	; search pattern
SrchFlags	dd	0		; flag combination
szNullMsg	db	"Nothing found in any Items!",0
		align	dword


; Group Dialog.

GDlgLVWidth	equ	200		; width of ListView


; Export Dialog.

hWaitCursor	dd	0		; handle, wait cursor


; Options Dialog.

; Color value storage for Options Dialog.
; Indexed in the order of the Time Color indexes (time.inc).

Colors		dd	nColors dup (0)

hOptnDlg	dd	0		; handle, Options Dialog
CCUp		dd	0		; flag, ChooseColor dialog up




; Object allocation [none reqd].

;SB_OAF		dd	0

; Flags, bit set indicates object/resource etc are allocated.
; These are used in the deallocation routine.




; SideBar module error handling.


; SideBar module error codes.

ERR_SB_Module	equ	1200h

ERR_SBInit	equ	ERR_SB_Module + 0	; (placeholder only)
ERR_WndReg	equ	ERR_SB_Module + 1
ERR_WndCreate	equ	ERR_SB_Module + 2
ERR_TBarCreate	equ	ERR_SB_Module + 3


; SBar_Init error messages.
;
; sERR_SBInit has one of the following error messages appended.

sERR_SBInit	db	"SideBar Init failed - ",0
sERR_WndReg	db	"Window registration",0
sERR_WndCreate	db	"Window create",0
sERR_TBarCreate	db	"ToolBar create",0

ErrorMsgs	dd	offset sERR_SBInit
		dd	offset sERR_WndReg
		dd	offset sERR_WndCreate
		dd	offset sERR_TBarCreate

nErrors		equ	4




; Tui Help.

include		sbarhlp.asm




.code




; Public functions.

include		sbarpub.asm




; Private functions.

option		proc:private




;----------------------------------------------------------------------------
SBarWndHdlr	proc	uses ebx edi esi,
			hWnd:dword, uMsg:dword, wParam:dword, lParam:dword
;----------
;
; SideBar Window message handler.
;----------------------------------------------------------------------------


	local	Point		:POINT


	mov	eax, uMsg

	cmp	eax, WM_TIMER
	jz	@@OnTimer
	cmp	eax, WM_NOTIFY
	jz	@@OnNotify
	cmp	eax, WM_COMMAND
	jz	@@OnCommand
	cmp	eax, WM_ACTIVATE
	jz	@@OnActivate
	cmp	eax, WM_CLOSE
	jz	@@OnClose
	cmp	eax, WM_DESTROY
	jz	@@OnDestroy
	cmp	eax, WM_CREATE
	jz	@@OnCreate

	jmp	@@DefHdlr


@@OnTimer:	;------------------------------------------------------------
		; Mouse Detect Timer.


	; If mouse has moved to right edge of screen then show SideBar.
	; If mouse has moved out of SideBar then hide it.

	.if	(wParam == MouseDetTimer)	; Ensure it is our timer

		invoke	GetCursorPos, addr Point
		mov	eax, Point.x

		mov	edx, G_ScrnWid
		dec	edx

		.if	(eax < SBarXLeft)

			invoke	ShowWindow, hWnd, SW_HIDE

		.elseif	(eax >= edx)

			invoke	ShowWindow, hWnd, SW_SHOW

		.endif

		jmp	@@MsgHandled

	.else

		jmp	@@DefHdlr

	.endif


@@OnNotify:	;------------------------------------------------------------
		; An event has occurred.


	mov	eax, lParam
	mov	ebx, eax

	; If ToolTip required then load and display it.

	.if	(TOOLTIPTEXT.hdr.code[ebx] == TTN_NEEDTEXT)

		mov	eax, TOOLTIPTEXT.hdr.idFrom[ebx]

		.if     (eax == ID_SB_BTTIME)

			mov  TOOLTIPTEXT.lpszText[ebx], offset szTime

		.elseif (eax == ID_SB_BTDESK)

			mov  TOOLTIPTEXT.lpszText[ebx], offset szDesk

		.elseif (eax == ID_SB_BTFIND)

			mov  TOOLTIPTEXT.lpszText[ebx], offset szFind

		.elseif (eax == ID_SB_BTGRP)

			mov  TOOLTIPTEXT.lpszText[ebx], offset szGroup

		.elseif (eax == ID_SB_BTTHUM)

			mov  TOOLTIPTEXT.lpszText[ebx], offset szThum

		.elseif (eax == ID_SB_BTEXPT)

			mov  TOOLTIPTEXT.lpszText[ebx], offset szExpt

		.elseif (eax == ID_SB_BTOPTN)

			mov  TOOLTIPTEXT.lpszText[ebx], offset szOpt

		.elseif (eax == ID_SB_BTHELP)

			mov  TOOLTIPTEXT.lpszText[ebx], offset szHelp

		.elseif (eax == ID_SB_BTEXIT)

			mov  TOOLTIPTEXT.lpszText[ebx], offset szExit

		.endif

	.endif

	jmp	@@MsgHandled


@@OnCommand:	;------------------------------------------------------------
		; Button pressed.


	movzx	eax, wPLo

	.if 	(eax == ID_SB_BTTIME)


		; Enable SideBar buttons.

		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTTIME, FALSE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTDESK, TRUE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTFIND, TRUE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTGRP, TRUE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTTHUM, TRUE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTEXPT, TRUE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTOPTN, TRUE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTHELP, TRUE

		; Show Time window and set it foreground to give slightly
		; higher priority and keyboard focus for Up/Dn keys.
		; On ensuing Time window activate, disable Database update.

		invoke	ShowWindow, hWnd, SW_HIDE	; (must do this)
		invoke	ShowWindow, L_hTimeWnd, SW_SHOW
		invoke	SetForegroundWindow, L_hTimeWnd


	.elseif	(eax == ID_SB_BTDESK)


		; Disable SideBar buttons.

		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTTIME, TRUE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTDESK, FALSE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTFIND, FALSE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTGRP, FALSE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTTHUM, FALSE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTEXPT, FALSE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTOPTN, FALSE
		invoke	SendMessageA, hTBWnd, TB_ENABLEBUTTON, ID_SB_BTHELP, FALSE

		; Minimise SideBar and Time window, Desktop window to top.
		; On ensuing Time window deactivate, enable Database update.

		invoke	ShowWindow, L_hTimeWnd, SW_HIDE
		invoke	ShowWindow, hWnd, SW_HIDE


	.elseif	(eax == ID_SB_BTFIND)


		; Find Dialog.

		invoke	DialogBoxParamA, G_hTui, ID_FN_DLG, hWnd,
				addr FindDlgHdlr, NULL

		; Always enable the Time window afterward.

		invoke	PostMessageA, hWnd, WM_COMMAND, ID_SB_BTTIME, NULL


	.elseif	(eax == ID_SB_BTGRP)


		; Group Dialog.

		invoke	DialogBoxParamA, G_hTui, ID_GR_DLG, hWnd,
				addr GrpDlgHdlr, NULL

		; Always enable the Time window afterward.

		invoke	PostMessageA, hWnd, WM_COMMAND, ID_SB_BTTIME, NULL


	.elseif	(eax == ID_SB_BTTHUM)


		; Toggle Thumbnail display on/off.

		invoke	Time_TogThumb


	.elseif	(eax == ID_SB_BTEXPT)


		; Export current Search.

		invoke	DialogBoxParamA, G_hTui, ID_EX_DLG, hWnd,
				addr ExptDlgHdlr, NULL

		; Always enable the Time window afterward.

		invoke	PostMessageA, hWnd, WM_COMMAND, ID_SB_BTTIME, NULL


	.elseif	(eax == ID_SB_BTOPTN)


		; Tui Options.

		invoke	DialogBoxParamA, G_hTui, ID_OP_DLG, hWnd,
				addr OptnDlgHdlr, NULL

		; Always enable the Time window afterward.

		invoke	PostMessageA, hWnd, WM_COMMAND, ID_SB_BTTIME, NULL


	.elseif	(eax == ID_SB_BTHELP)


		; Tui Help.

		invoke	DialogBoxParamA, G_hTui, ID_HP_DLG, hWnd,
				addr HelpDlgHdlr, NULL

		; Always enable the Time window afterward.

		invoke	PostMessageA, hWnd, WM_COMMAND, ID_SB_BTTIME, NULL


	.elseif	(eax == ID_SB_BTEXIT)


		; Destroy Time and SideBar windows. These will remove the
		; windows and send WM_DESTROY and WM_NCDESTROY to them
		; before exiting back here. The WM_DESTROY to the SideBar
		; window is picked up below where we post WM_QUIT.

		invoke	Time_Destroy
		invoke	SBar_Destroy

		jmp	@@MsgHandled


	.endif


	invoke	InvalidateRect, hWnd, 0, TRUE		; Update window

	jmp	@@MsgHandled


@@OnActivate:	;------------------------------------------------------------
		; Activate/DeActivate.


	movzx	eax, wPLo			; Activation flag

	.if	(eax == WA_INACTIVE)

		if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_SB_WMAct, FALSE, lParam
		endif		;++++++++++++++++++++++++++++++++++++++++++++

	.else

		if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_SB_WMAct, TRUE, lParam
		endif		;++++++++++++++++++++++++++++++++++++++++++++

	.endif

	jmp	@@MsgHandled


@@OnClose:	;------------------------------------------------------------
		; Close window.


	; Do nothing, thus Alt-F4 is disabled.

	jmp	@@MsgHandled


@@OnDestroy:	;------------------------------------------------------------
		; Destroy window.


	invoke	KillTimer, hWnd, MouseDetTimer


	; Exit TUI message loop (tui.asm).

	invoke  PostQuitMessage, NULL

	jmp	@@MsgHandled


@@OnCreate:	;------------------------------------------------------------
		; Create window.


	; Start the detection timer, 100mS period.

	invoke	SetTimer, hWnd, MouseDetTimer, 100, 0


	; We start with the Time window up.
	;
	; It was found to be better to post the BTTIME message rather than
	; show the window here as otherwise an inactivate message is sent to
	; the Time window after the activate message(?).

	invoke	PostMessageA, hWnd, WM_COMMAND, ID_SB_BTTIME, NULL

	jmp	@@MsgHandled


@@MsgHandled:	;------------------------------------------------------------


	xor	eax,  eax

	ret


@@DefHdlr:	;------------------------------------------------------------


	invoke	DefWindowProcA, hWnd, uMsg, wParam, lParam

	ret


SBarWndHdlr	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
FindDlgHdlr	proc	uses ebx esi edi,
			hWnd:dword,uMsg:dword,wParam:dword,lParam :dword
;----------
;
; Find Dialog message handler.
;
;
; Exit		!0	Message processed
;
;		0	Message not processed - the dialog manager performs
;			the default dialog operation for the message.
;
;		End	NULL
;----------------------------------------------------------------------------


	mov	eax, uMsg

	cmp	eax, WM_INITDIALOG
	jz	@@OnInit
	cmp	eax, WM_COMMAND
	jz	@@OnCommand
	cmp	eax, WM_ACTIVATE
	jz	@@OnActivate
	cmp	eax, WM_CLOSE
	jz	@@OnClose

	xor	eax, eax			; Msg not processed
	jmp	@@ExitHandler


@@OnInit:	;------------------------------------------------------------
		; Initialise the Dialog box.


	invoke	LoadCursorA, NULL, IDC_WAIT	; for long searches
	mov	hWaitCursor, eax


	; Initialise the controls and set states to prior (if any).

	.if	(bChkCase == 1)
		invoke	CheckDlgButton, hWnd, ID_FN_CKCASE, BST_CHECKED
	.else
		invoke	CheckDlgButton, hWnd, ID_FN_CKCASE, BST_UNCHECKED
	.endif

	.if	(bChkNameEnd == 1)
		invoke	CheckDlgButton, hWnd, ID_FN_CKNEND, BST_CHECKED
	.else
		invoke	CheckDlgButton, hWnd, ID_FN_CKNEND, BST_UNCHECKED
	.endif

	.if	(bChkInData)

		mov	eax, ID_FN_RDDATA

	.elseif	(bChkInPath)

		mov	eax, ID_FN_RDPATH

	.else	; Default.

		mov	bChkInName, 1
		mov	eax, ID_FN_RDNAME

	.endif

	invoke	CheckRadioButton, hWnd, ID_FN_RDNAME, ID_FN_RDPATH, eax


	; Show the last search text (if any).

	invoke	SendDlgItemMessage, hWnd, ID_FN_EDIT, WM_SETTEXT, 0,
			addr SrchText

	jmp	@@MsgHandled


@@OnCommand:	;------------------------------------------------------------
		; Mouse click on buttons.


	movzx	eax, wPLo


	.if	(eax == ID_FN_RDNAME)


		; Toggle RadioButton state.

		invoke	CheckRadioButton, hWnd, ID_FN_RDNAME,
						ID_FN_RDPATH, eax


	.elseif	(eax == ID_FN_RDDATA)


		; Toggle RadioButton state.

		invoke	CheckRadioButton, hWnd, ID_FN_RDNAME,
						ID_FN_RDPATH, eax

		invoke	IsDlgButtonChecked, hWnd, ID_FN_RDDATA

			; If checked turn off Name End switch.

			.if	(eax == BST_CHECKED)

				invoke	CheckDlgButton, hWnd, ID_FN_CKNEND,
						BST_UNCHECKED

			.endif

	.elseif	(eax == ID_FN_RDPATH)


		; Toggle RadioButton state.

		invoke	CheckRadioButton, hWnd, ID_FN_RDNAME,
						ID_FN_RDPATH, eax

		invoke	IsDlgButtonChecked, hWnd, ID_FN_RDPATH

			; If checked turn off Name End switch.

			.if	(eax == BST_CHECKED)

				invoke	CheckDlgButton, hWnd, ID_FN_CKNEND,
						BST_UNCHECKED

			.endif


	.elseif	(eax == IDOK)


		; Get user text.

		invoke	SendDlgItemMessage, hWnd, ID_FN_EDIT,
				WM_GETTEXTLENGTH, 0, 0

		mov	SrchTxtLen, eax


		; Only action if user has typed something in.

		.if	(eax)


			; Show the wait cursor.

			invoke	SetCursor, hWaitCursor


			; Get the state of the various option buttons,
			; checkboxes.

			invoke	IsDlgButtonChecked, hWnd, ID_FN_CKCASE

			.if	(eax == BST_CHECKED)
				mov	bChkCase, 1
				or	SrchFlags, I_SRCHCASE
			.else
				mov	bChkCase, 0
				and	SrchFlags, NOT I_SRCHCASE
			.endif

			invoke	IsDlgButtonChecked, hWnd, ID_FN_CKNEND

			.if	(eax == BST_CHECKED)
				mov	bChkNameEnd, 1
				or	SrchFlags, I_SRCHNEND
			.else
				mov	bChkNameEnd, 0
				and	SrchFlags, NOT I_SRCHNEND
			.endif

			invoke	IsDlgButtonChecked, hWnd, ID_FN_RDNAME

			.if	(eax == BST_CHECKED)
				mov	bChkInName, 1
				or	SrchFlags, I_SRCHNAME
			.else
				mov	bChkInName, 0
				and	SrchFlags, NOT I_SRCHNAME
			.endif

			invoke	IsDlgButtonChecked, hWnd, ID_FN_RDDATA

			.if	(eax == BST_CHECKED)
				mov	bChkInData, 1
				or	SrchFlags, I_SRCHDATA
			.else
				mov	bChkInData, 0
				and	SrchFlags, NOT I_SRCHDATA
			.endif

			invoke	IsDlgButtonChecked, hWnd, ID_FN_RDPATH

			.if	(eax == BST_CHECKED)
				mov	bChkInPath, 1
				or	SrchFlags, I_SRCHPATH
			.else
				mov	bChkInPath, 0
				and	SrchFlags, NOT I_SRCHPATH
			.endif

			mov	eax, SrchTxtLen
			inc	eax			; include terminator

			invoke	SendDlgItemMessage, hWnd, ID_FN_EDIT,
					WM_GETTEXT, eax, addr SrchText


			; Do the search.

			.if	(SrchText[0] == '*')	; find everything
				mov	edx, I_SRCHRALL
			.else
				mov	edx, SrchFlags
			.endif

			invoke	DB_Search, edx, addr SrchText, SrchTxtLen

			.if	(eax)

				; If Items found then reset the Time display
				; and exit.

				invoke	Time_SetItem, TSetFirst, 0, eax

				invoke	PostMessageA, hWnd, WM_CLOSE,
						NULL, NULL

			.else

				; Else notify user and return to try again.

				invoke	SendDlgItemMessage, hWnd, ID_FN_EDIT,
						WM_SETTEXT, 0, addr szNullMsg

			.endif


		.endif


	.elseif	(eax == IDCANCEL)

		invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL

	.else

		; Message unhandled.

		xor	eax, eax
		jmp	@@ExitHandler

	.endif

	jmp	@@MsgHandled


@@OnActivate:	;------------------------------------------------------------
		; Activate/DeActivate.


	movzx	eax, wPLo			; Activation flag

	.if	(eax == WA_INACTIVE)

		; Close the dialog if the user has clicked outside it.

		invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL

		if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_FD_WMAct, FALSE, lParam
		endif		;++++++++++++++++++++++++++++++++++++++++++++

	.else

		; Spin here until a database update is finished.

		invoke	WaitForSingleObject, G_UDBhMutex, INFINITE
		invoke	ReleaseMutex, G_UDBhMutex

		if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_FD_WMAct, TRUE, lParam
		endif		;++++++++++++++++++++++++++++++++++++++++++++

	.endif

	jmp	@@MsgHandled


@@OnClose:	;------------------------------------------------------------
		; Close Dialog box, Alt-F4 etc.


	invoke	EndDialog, hWnd, 0


@@MsgHandled:	;------------------------------------------------------------
		; Exit, Dialog message handled.


	mov	eax, TRUE


@@ExitHandler:


	ret


FindDlgHdlr	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
GrpDlgHdlr	proc	uses ebx esi edi,
			hWnd:dword,uMsg:dword,wParam:dword,lParam :dword
;---------
;
; Group Dialog message handler.
;
;
; Exit		!0	Message processed
;
;		0	Message not processed - the dialog manager performs
;			the default dialog operation for the message.
;
;		End	NULL
;----------------------------------------------------------------------------


	local	GMask			:dword		; Group mask


	mov	eax, uMsg

	cmp	eax, WM_INITDIALOG
	jz	@@OnInit
	cmp	eax, WM_COMMAND
	jz	@@OnCommand
	cmp	eax, WM_ACTIVATE
	jz	@@OnActivate
	cmp	eax, WM_CLOSE
	jz	@@OnClose

	xor	eax, eax			; Msg not processed
	jmp	@@ExitHandler


@@OnInit:	;------------------------------------------------------------
		; Initialise the Dialog box.


	; Initialise Group ListView.

	invoke	InitGrpLV, hWnd

	jmp	@@MsgHandled


@@OnCommand:	;------------------------------------------------------------
		; Mouse click on buttons/controls.


	movzx	eax, wPLo


	.if	(eax == IDOK)


		; Assemble a bit mask of the Groups selected.

		xor	ebx, ebx
		mov	GMask, ebx

		.repeat

			; Is Group selected?.

			invoke	SendDlgItemMessage, hWnd, ID_GR_LSVGRP,
					LVM_GETITEMSTATE, ebx,
					LVIS_STATEIMAGEMASK

			shr	eax, 12
			dec	eax

			.if	(eax)

				; Yes, set Group mask bit.

				bts	GMask, ebx

			.endif

			inc	ebx

		.until	(ebx == kGroups)


		; If nothing selected then go round again.

		.if	(!GMask)
			jmp	@@MsgHandled
		.endif


		; Tell the Database module which Groups are selected.

		invoke	DB_GroupMask, TRUE, GMask


		; Search on everything and reset the Time display.

		invoke	DB_Search, I_SRCHRALL, NULL, NULL

		.if	(eax)
			invoke	Time_SetItem, TSetFirst, 0, eax
		.endif

		jmp	@@CloseDialog


	.elseif	(eax == IDCANCEL)


		@@CloseDialog:

		invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL


	.else

		; Message unhandled.

		xor	eax, eax
		jmp	@@ExitHandler

	.endif

	jmp	@@MsgHandled


@@OnActivate:	;------------------------------------------------------------
		; Activate/DeActivate.


	movzx	eax, wPLo			; Activation flag

	.if	(eax == WA_INACTIVE)

		; Close the dialog if the user has clicked outside it.

		invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL

		if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_GP_WMAct, FALSE, lParam
		endif		;++++++++++++++++++++++++++++++++++++++++++++

	.else

		; Spin here until a database update is finished.

		invoke	WaitForSingleObject, G_UDBhMutex, INFINITE
		invoke	ReleaseMutex, G_UDBhMutex

		if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_GP_WMAct, TRUE, lParam
		endif		;++++++++++++++++++++++++++++++++++++++++++++

	.endif

	jmp	@@MsgHandled


@@OnClose:	;------------------------------------------------------------
		; Close Dialog box, Alt-F4 etc.


	invoke	EndDialog, hWnd, 0


@@MsgHandled:	;------------------------------------------------------------
		; Exit, Dialog message handled


	mov	eax, TRUE


@@ExitHandler:


	ret


GrpDlgHdlr	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
InitGrpLV	proc	uses ebx, hWnd:dword
;--------
;
; Initialise the Group Dialog ListView for user input.
;
; The ListView has one column displaying the available Groups, each with a
; CheckBox for group selection.
;
;
; Entry		hWnd		Group Dialog handle
;
;
; Exit		n/a
;----------------------------------------------------------------------------


	local	lvc			:LV_COLUMN
	local	lvi			:LV_ITEM
	local	GMask			:dword		; Group mask


; Add Checkbox to each Item and create 1 column (# 0).
;
; Note the column width is in pixels which don't correspond to dialog units.


	invoke	SendDlgItemMessage, hWnd, ID_GR_LSVGRP,
			LVM_SETEXTENDEDLISTVIEWSTYLE, 0, LVS_EX_CHECKBOXES

	mov	lvc.imask, LVCF_WIDTH
	mov	lvc.lx, GDlgLVWidth

	invoke	SendDlgItemMessage, hWnd, ID_GR_LSVGRP,
			LVM_INSERTCOLUMN, 0, addr lvc


; Load the ListView with the currently selected Groups.


	invoke	DB_GroupMask, FALSE, NULL
	mov	GMask, eax

	xor	ebx, ebx

	.repeat

		; Insert the Group name string to column 0.

		mov	eax, ebx
		inc	eax			; Groups indexed 1 up
		invoke	DB_GetGrpName, eax
		mov	lvi.pszText, eax
		mov	lvi.imask, LVIF_TEXT
		mov	lvi.iItem, ebx
	        xor	eax, eax
		mov	lvi.lParam, eax
		mov	lvi.iSubItem, eax
		mov	lvi.iImage, eax

		invoke	SendDlgItemMessage, hWnd, ID_GR_LSVGRP,
				LVM_INSERTITEM, 0, addr lvi

		; Set the current Group select state.
		; (see \doc\listview.txt)

		bt	GMask, ebx

		.if	(CARRY?)
			mov	eax, 2		; checked CheckBox
		.else
			mov	eax, 1		; unchecked CheckBox
		.endif

		shl	eax, 12
		mov	lvi.state, eax
		mov	lvi.stateMask, LVIS_STATEIMAGEMASK
		mov	lvi.imask, LVIF_STATE

		invoke	SendDlgItemMessage, hWnd, ID_GR_LSVGRP,
				LVM_SETITEM, 0, addr lvi

		inc	ebx

	.until	(ebx == kGroups)


	ret


InitGrpLV	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
ExptDlgHdlr	proc	uses ebx esi edi,
			hWnd:dword,uMsg:dword,wParam:dword,lParam :dword
;----------
;
; Export Search Dialog message handler.
;
;
; Exit		!0	Message processed
;
;		0	Message not processed - the dialog manager performs
;			the default dialog operation for the message.
;
;		End	NULL
;----------------------------------------------------------------------------


	mov	eax, uMsg

	cmp	eax, WM_INITDIALOG
	jz	@@OnInit
	cmp	eax, WM_COMMAND
	jz	@@OnCommand
	cmp	eax, WM_ACTIVATE
	jz	@@OnActivate
	cmp	eax, WM_CLOSE
	jz	@@OnClose

	xor	eax, eax			; Msg not processed
	jmp	@@ExitHandler


@@OnInit:	;------------------------------------------------------------
		; Initialise the Dialog box.


	invoke	LoadCursorA, NULL, IDC_WAIT	; for a big database
	mov	hWaitCursor, eax

	jmp	@@MsgHandled


@@OnCommand:	;------------------------------------------------------------
		; Mouse click on buttons/controls.


	movzx	eax, wPLo

	.if	(eax == IDOK)


		invoke	SetCursor, hWaitCursor	; Show hourglass

		invoke	DB_Export

		jmp	@@CloseDialog


	.elseif	(eax == IDCANCEL)


		@@CloseDialog:

		invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL


	.endif

	jmp	@@MsgHandled


@@OnActivate:	;------------------------------------------------------------
		; Activate/DeActivate.



	movzx	eax, wPLo			; Activation flag

	.if	(eax == WA_INACTIVE)

		; Close the dialog if the user has clicked outside it.

		invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL

		if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_EX_WMAct, FALSE, lParam
		endif		;++++++++++++++++++++++++++++++++++++++++++++

	.else

		; Spin here until a database update is finished.

		invoke	WaitForSingleObject, G_UDBhMutex, INFINITE
		invoke	ReleaseMutex, G_UDBhMutex

		if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_EX_WMAct, TRUE, lParam
		endif		;++++++++++++++++++++++++++++++++++++++++++++

	.endif

	jmp	@@MsgHandled


@@OnClose:	;------------------------------------------------------------
		; Close Dialog box, Alt-F4 etc.


	invoke	EndDialog, hWnd, 0


@@MsgHandled:	;------------------------------------------------------------
		; Exit, Dialog message handled.


	mov	eax, TRUE


@@ExitHandler:


	ret


ExptDlgHdlr	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
OptnDlgHdlr	proc	uses ebx esi edi,
			hWnd:dword,uMsg:dword,wParam:dword,lParam :dword
;----------
;
; Options Dialog message handler.
;
;
; Exit		!0	Message processed
;
;		0	Message not processed - the dialog manager performs
;			the default dialog operation for the message.
;
;		End	NULL
;----------------------------------------------------------------------------


	mov	eax, uMsg

	cmp	eax, WM_INITDIALOG
	jz	@@OnInit
	cmp	eax, WM_DRAWITEM
	jz	@@OnDrawItem
	cmp	eax, WM_COMMAND
	jz	@@OnCommand
	cmp	eax, WM_ACTIVATE
	jz	@@OnActivate
	cmp	eax, WM_CLOSE
	jz	@@OnClose

	xor	eax, eax			; Msg unhandled
	jmp	@@ExitHandler


@@OnInit:	;------------------------------------------------------------
		; Initialise the Dialog box.


	; We need our dialog's handle.

	mov	eax, hWnd
	mov	hOptnDlg, eax

	; Get the existing Time color values.

	invoke	Time_Color, GetColors, addr Colors

	jmp	@@MsgHandled


@@OnDrawItem:	;------------------------------------------------------------
		; OwnerDrawn buttons are to be drawn.


	; Fill Color button backgrounds with their colors.

	mov	eax, wParam			; Button ID
	mov	ebx, lParam			; DRAWITEMSTRUCT

	.if	(eax ==	ID_OP_BTTWBG)

		invoke	CreateSolidBrush, Colors[TC_TWBG*4]
		push	eax

	.elseif	(eax == ID_OP_BTITBG)

		invoke	CreateSolidBrush, Colors[TC_ITBG*4]
		push	eax

	.elseif	(eax == ID_OP_BTTSTX)

		invoke	CreateSolidBrush, Colors[TC_TSTX*4]
		push	eax

	.elseif	(eax == ID_OP_BTTBBG)

		invoke	CreateSolidBrush, Colors[TC_TBBG*4]
		push	eax

	.endif

	invoke	FillRect, [ebx + DRAWITEMSTRUCT.hdc],
			addr [ebx + DRAWITEMSTRUCT.rcItem], eax

	pop	eax
	invoke	DeleteObject, eax

	jmp	@@MsgHandled


@@OnCommand:	;------------------------------------------------------------
		; Mouse click on buttons/controls.


	movzx	eax, wPLo


	.if	(eax == ID_OP_BTTWBG)		; Time Window background

		mov	eax, Colors[TC_TWBG*4]
		invoke	SetColor, hWnd, TC_TWBG, eax

		jmp	@@MsgHandled

	.elseif	(eax == ID_OP_BTITBG)		; Item Background

		mov	eax, Colors[TC_ITBG*4]
		invoke	SetColor, hWnd, TC_ITBG, eax

		jmp	@@MsgHandled

	.elseif	(eax == ID_OP_BTTSTX)		; TimeStamp text

		mov	eax, Colors[TC_TSTX*4]
		invoke	SetColor, hWnd, TC_TSTX, eax

		jmp	@@MsgHandled

	.elseif	(eax == ID_OP_BTTBBG)		; Time Box background

		mov	eax, Colors[TC_TBBG*4]
		invoke	SetColor, hWnd, TC_TBBG, eax

		jmp	@@MsgHandled

	.elseif	(eax == IDOK)


		; Set the new Time color values.

		invoke	Time_Color, SetColors, addr Colors

		jmp	@@CloseDialog


	.elseif	(eax == IDCANCEL)


		@@CloseDialog:

		invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL


	.endif

	jmp	@@MsgHandled


@@OnActivate:	;------------------------------------------------------------
		; Activate/DeActivate.


	; If the Options dialog is losing focus and the ChooseColor dialog
	; is not up then the user has clicked outside the Options dialog so
	; shut it down. See the ChooseColor hook procedure below also.

	movzx	eax, wPLo			; Activation flag

	.if	(eax == WA_INACTIVE && !CCUp)

		invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL

	.endif

	jmp	@@MsgHandled


@@OnClose:	;------------------------------------------------------------
		; Close Dialog box, Alt-F4 etc.


	invoke	EndDialog, hWnd, 0


@@MsgHandled:	;------------------------------------------------------------
		; Exit, Dialog message handled.


	mov	eax, TRUE


@@ExitHandler:


	ret



OptnDlgHdlr	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
SetColor	proc	hWnd:dword,Ndx:dword,CurrColor:dword
;-------
;
; Select and set a color value in the Colors array.
;
;
; Entry		hWnd		handle, parent window
;		Ndx		index of color value into the Colors array
;		CurrColor	current color value
;
;		External:
;
;		Colors[]	color value array
;		CCUp		flag, true if ChooseColor dialog is up
;
;
; Exit		Colors[Ndx]	updated
;----------------------------------------------------------------------------


	local	cc		:CHOOSECOLORA
	local	cca[16]		:dword		; custom color array


; Set dialog up flag and clear custom color array.
;
; We don't use custom colors but we must provide the space jic user selects.


	mov	CCUp, 1

	lea	edi, cca
	mov	ecx, 16
	mov	eax, 0FFFFFFh
	rep	stosd


; Initialise structure and get color.


	mov	cc.lStructSize, sizeof CHOOSECOLORA
	push	hWnd
	pop	cc.hwndOwner
	mov	cc.hInstance, 0
	push	CurrColor
	pop	cc.rgbResult
	lea	eax, cca
	mov	cc.lpCustColors, eax
	mov	cc.Flags, CC_RGBINIT or CC_FULLOPEN or CC_ENABLEHOOK
	mov	cc.lCustData, 0
	mov	cc.lpfnHook, offset CCDlgHdlr
	mov	cc.lpTemplateName, 0

	invoke ChooseColorA, addr cc


; If a color value is returned then load it.


	.if	(eax)

		mov	edx, Ndx
		mov	eax, cc.rgbResult
		mov	Colors[edx*4], eax

	.endif


; Force windows to repaint Options dialog to show the new color,
; reset dialog up flag and exit.


	invoke	InvalidateRect, hWnd, NULL, NULL

	mov	CCUp, 0


	ret

SetColor	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
CCDlgHdlr	proc	uses ebx esi edi,
			hWnd:dword,uMsg:dword,wParam:dword,lParam:dword
;--------
;
; ChooseColor Dialog hook message handler.
;
; We need to hook WM_ACTIVATE to filter a user click outside the dialog. Then
; we can close both it and the Options dialog to prevent them disappearing
; behind the Time Window with the SideBar disabled.
;
;
; Exit		!0	The default CC dialog procedure ignores the message
;
;		0	The default CC dialog procedure processes the message
;
;		End	NULL
;----------------------------------------------------------------------------


	mov	eax, uMsg

	.if	(eax == WM_ACTIVATE)

		movzx	eax, wPLo		; Activation flag
		mov	edx, lParam		; handle, window 2b activated

		.if	(eax == WA_INACTIVE)

			; The ChangeColor dialog is losing the focus. If the
			; user has clicked outside the dialog, in which case
			; the window being activated will not be the Options
			; dialog, then we need to close the Options dialog
			; as well.

			.if	(edx != hOptnDlg)

				invoke	PostMessageA, hOptnDlg, WM_CLOSE,
						NULL, NULL

			.endif

			invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL

		.endif

		mov	eax, TRUE		; Msg processed

	.else

		xor	eax, eax		; Msg not processed

	.endif


	ret


CCDlgHdlr	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
HelpDlgHdlr	proc	uses ebx esi edi,
			hWnd:dword,uMsg:dword,wParam:dword,lParam :dword
;----------
;
; Help Dialog message handler.
;
;
; Exit		!0	Message processed
;
;		0	Message not processed - the dialog manager performs
;			the default dialog operation for the message.
;
;		End	NULL
;----------------------------------------------------------------------------


	mov	eax, uMsg

	cmp	eax, WM_INITDIALOG
	jz	@@OnInit
	cmp	eax, WM_CTLCOLORSTATIC
	jz	@@OnColor
	cmp	eax, WM_ACTIVATE
	jz	@@OnActivate
	cmp	eax, WM_CLOSE
	jz	@@OnClose

	xor	eax, eax			; Msg not processed
	jmp	@@ExitHandler


@@OnInit:	;------------------------------------------------------------
		; Initialise the Dialog box.


	; Show the help text.

	invoke	SendDlgItemMessage, hWnd, ID_HP_EDIT, WM_SETTEXT, 0,
			addr HelpText

	jmp	@@MsgHandled


@@OnColor:	;------------------------------------------------------------
		; Edit control to be drawn.


	;"A static control, or an edit control that is read-only or disabled,
	; sends the WM_CTLCOLORSTATIC message to its parent window when the
	; control is about to be drawn. By responding to this message, the
	; parent window can use the specified device context handle to set the
	; text and background colors of the static control.
	;
	;	wParam		handle to display context
	;	lParam		handle to static control
	;
	; If an application processes this message, it must return the handle
	; of a brush. Windows uses the brush to paint the background of the
	; edit control."

	; USE this to color background to white.
	invoke	GetStockObject, WHITE_BRUSH
	; OR
	; This to keep as standard gray.
	;xor	eax, eax

	jmp	@@ExitHandler


@@OnActivate:	;------------------------------------------------------------
		; Activate/DeActivate.


	; Deselect text as otherwise annoyingly selected!. Quite why we need
	; do do this here is not obvious - does not work at WM_INITDIALOG.

	invoke	SendDlgItemMessage, hWnd, ID_HP_EDIT, EM_SETSEL, -1, -1

	movzx	eax, wPLo			; Activation flag

	.if	(eax == WA_INACTIVE)

		; Close the dialog if the user has clicked outside it.

		invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL

	.endif

	jmp	@@MsgHandled


@@OnClose:	;------------------------------------------------------------
		; Close Dialog box, Alt-F4 etc.


	invoke	EndDialog, hWnd, 0


@@MsgHandled:	;------------------------------------------------------------
		; Exit, Dialog message handled.


	mov	eax, TRUE


@@ExitHandler:


	ret


HelpDlgHdlr	endp
;----------------------------------------------------------------------------




end		; Sidebar module.




;============================================================================
