;============================================================================
; TIME.ASM
;
; Time display module.
;
; Attention: This module, its functions and associated variables etc are
; finicky, tricky and took way too long to get to work properly!.
; If modifying it test carefully and often.
;
; The Time window message handler primarily looks after movement of the Time
; window. The SideBar module message handler looks after management of the
; Time window state.
;
; Note that files are seen as 'Items' in the Time display.
;============================================================================




.nolist

include		..\tui.inc
include		..\comn\comn.inc
include		..\db\db.inc
include		..\sbar\sbar.inc

.listall

include		time.inc




; Private functions.

; timeshow.asm

ShowTime	proto
DrawItem	proto	x:dword,y:dword,x:dword,hDC:dword,Item:dword

; Below.

TimeWndHdlr	proto	hWnd:dword,uMsg:dword,wParam:dword,lParam:dword
SetTimeParams	proto	TxtHgt:dword
QryInLargeItem	proto
EditDlgHdlr	proto	hWnd:dword,uMsg:dword,wParam:dword,lParam:dword




; Automatic Move Timer.

AutoMoveTimer	equ	1
MoveTmrPeriod	equ	33			; mS (~30fps)




; Default colours.
;
; 0x00BBGGRR (api COLORREF format)

DefCTime	equ	00BBBBBBh		; Time Window background
DefCItem	equ	0090F0F0h		; Item background
DefCTS		equ	00FF0000h		; TimeStamp text
DefTimeBox	equ	CYellow			; Time Box background




.data		; Private to Time module.




; Time Window.

szTimeClass	db	"Time", 0		; Time window class name
		align	dword

hTimeWnd	dd	0			; handle, Time window


; Default and maximum number of items displayed in Time window.
;
; Note that this directly affects what size items are displayed and the
; cutoff points for variable sized items (see below and DrawItem).

if ShowTDisp	;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 DefDispItems	equ	10			; (Meaningless if > ItemVarZ)
else		;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

 DefDispItems	equ	50

endif




; Time window parameters.


xMid		dd	0			; Midpoint of screen width
yMid		dd	0			; Midpoint of screen height
SrchNdx		dd	0			; Item Search array ndx (1 up)
DispItems	dd	DefDispItems		; Items displayed
SrchItems	dd	0			; Total items in current srch
BaseItem	dd	0			; Base Item in Time display
CurrItem	dd	0			; Current 1st Item in display

ScrnRect	RECT	<>			; client rect (screen size)
hDC		dd	0			; handle, client rectangle DC
hMemDC		dd	0			; handle, compatible DC
hBitmap		dd	0			; handle, compatible bitmap

hBGBrush	dd	0			; handle, Time window bg brush
hWhiteBrush	dd	0			; handle, white brush
hItemBrush	dd	0			; handle, Item fill brush clr
hFont		dd	0			; handle, normal font
hFontB		dd	0			; handle, bold font

CTimeWnd	dd	0			; Time Window background
CTimeItem	dd	0			; Item background
CTimeStamp	dd	0			; TimeStamp text
CTimeBox	dd	0			; Time Box background

bMouseTop	dd	0			; flag, mouse at top of wnd
bMouseBot	dd	0			; flag, mouse at bot of wnd

bTimeStop	dd	0			; flag, movement disabled


; These set the 'spread' of the items as they come close to the user.
; NB these must be in this order - see ShowTime x calculation.

xI_R		real4	+0.6			; Right Item x
xI_L		real4	-0.6			; Left Item x

; Z parameters.

zAdj		real4	+1.0			; z adjustment value
zT		real4	+0.08			; Timer inc/dec per move
zDT		real4	+0.01			; Timer inc/dec delta
zM		real4	+0.5			; Mouse inc/dec per move

if ShowTDisp	;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
zW		real4	+0.01			; debug display
else		;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

zW		real4	+0.25			; Wheel inc/dec per move

endif

; Variable sized Item perspective constant.
;
; Adjusts size down to single pixel when reaching ItemVarZ.

zKVarItem	real4	+0.3




; DrawTime time boxen and status display.


; Various.

bMovTimeMov	dd	0			; flag, moving MovTime box
TBoxHgt		dd	0			; box height in pixels
TBoxHgt_1	dd	0			; height - 1, useful
TMRange		dd	0			; Time box range in pixels
hUpIcon		dd	0			; handle, Up icon
hDnIcon		dd	0			; handle, Dn icon


; Time boxes (DrawTime).
;
; These are initialised in Time_Init and TimeWndHdlr, WM_CREATE.
; A space is required at the start of each TSStrg field.
;
; The MovTime TSStrg field is initialised with a dummy maximum width string
; (+ an unwritten space for a divider line) for finding its maximum width
; so to 'append' the up/down icons (TimeWndHdlr/WM_CREATE).

TSstruc		struc
  left		dword	0			; coordinates
  top		dword	0
  right		dword	0
  bottom	dword	0
  TS		dword	0			; TimeStamp
  TSStrg	db	24 dup (0)		; TimeStamp string
TSstruc		ends

EndTime		TSstruc	<0,0,0,0,0,' '>
StaTime		TSstruc	<0,0,0,0,0,' '>
MovTime		TSstruc	<0,0,0,0,0,' 00/00/0000 00:00:00  '>
		align	dword

; Move Up/Dn icons.

MoveIcoXY	equ	16			; Icons are 16x16 pixels

; Saved state of current display of Search Array.

SA_FItem	dd	0			; First Item in Search array
SA_CItem	dd	0			; Current Item displayed
SA_NItems	dd	0			; Search Items

; Item number box.

ItemsBox	RECT	<0,0,0,0>




; Time display font.
;
; Note the lfHeight needs to be set negative for XP to properly display it.
;
; Quote from API doc:
;
;	(for lfHeight < 0)
;
;	"The font mapper transforms this value into device units and
;	matches its absolute value against the character height of the
;	available fonts."

FontSpec	LOGFONT <-10,0,0,0,FW_NORMAL,0,0,0,DEFAULT_CHARSET,\
			OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,\
			DEFAULT_QUALITY,DEFAULT_PITCH or FF_SWISS,\
			'MS Sans Serif'>
		align	dword




; Item dimensioning (DrawItem).

ItemVarZ	equ	30			; if z < then variable size
ItemLrgZ	equ	10			; if z < then large size
ItemThumb	equ	5			; if z < then show thumbnail
						; (see below at Thumbnails)

ItemWidL	equ	200			; Item width, large
ItemHgtL	equ	200			; Item height, large

ItemIcoXY	equ	32			; Icons are 32x32 pixels
ItemIcoX	equ	5			; Icon offset from top
ItemIcoY	equ	5			;   left of Item.
ItemTsX		equ	40			; TS offset from top left
						; (ItemIcoXY + ItemIcoX + pad)

ItemTxtY1	equ	16			; Interline text spacing
ItemTxtY2	equ	20			; Interline text spacing

ItemThmOff	equ	20			; Thumb offset from Path line
ItemThmWid	equ	ItemWidL-20		; Thumbnail width
ItemThmHgt	equ	ItemHgtL-70		; Thumbnail height


; Large Item specifications.
;
; NB code in DrawItem depends on the specific size of 20 bytes!.

LIstruc		struc
  Coord		RECT	<>			; coordinates
  Ndx		dd	0			; Search array index
LIstruc		ends

nLgItems	equ	ItemLrgZ-1		; total Large Items
LI		LIstruc	nLgItems dup (<>)	; Large Item array
LINdx		dd	0			; current index into LI (1 up)




; Thumbnailing.


DoThumb		dd	1			; Thumbnail enable, true/false


; Image Thumbnailing.


hGdipGraObj	dd	0			; handle, GDI+ graphics object

; Image object array.
;
; This provides buffering of four images to prevent constant reloading.
; The images are held internally by the GDI+ subsystem and are loaded at
; Time window WM_CREATE.

ImgObjStruc	struc
  Item		dd	0			; Item index
  hGra		dd	0			; handle, graphics object
  hBmp		dd	0			; handle, bitmap
  ImgXY		dd	0			; image dimensions in pixels:
ImgObjStruc	ends				; hi word = x, low = y

ImgObjRecSize	equ	sizeof ImgObjStruc	; 16 bytes
ImgObjRecMult	equ	4			; *16

; NB nImgObj must be a binary multiple as we use nImgObj-1 as a
; bit mask of an Item number in DrawItem Thumbnailing.

nImgObj		equ	ItemThumb-1		; (index 0..ItemThumb-2)

; Initialise with Item = -1 to indicate no image loaded (Items are 0 up).

ImgObj		ImgObjStruc nImgObj dup (<-1,0,0,0>)


; Text Thumbnailing.


nCharTxtThumb	equ	128			; #char in text thumbnail
nCharTTMult	equ	7			; *128

; Text object array (see Image object array above).
;
; This provides buffering of four thumbs to prevent constant reloading.

TxtObjStruc	struc
  Item		dd	0			; Item index
  Len		dd	0			; text length
TxtObjStruc	ends

TxtObjRecSize	equ	sizeof TxtObjStruc	; 8 bytes
TxtObjRecMult	equ	3			; *8

nTxtObj		equ	ItemThumb-1		; (index 0..ItemThumb-2)

TxtObj		TxtObjStruc nTxtObj dup (<-1>)

TxtObjArr	db	nCharTxtThumb*nTxtObj dup (0)




; Item Properties Dialog.


; Item right-click popup menu.

hPupMenu	dd	0			; handle, Popup Menu
MenuedItem	dd	0			; Item Menu is up for


; Menu item IDs.

IDM_PATH	equ	1
IDM_SIZE	equ	2
IDM_SPEC	equ	3
IDM_GOTO	equ	4
IDM_EDIT	equ	5
IDM_EXIT	equ	6


; Dummy string for initialising menu Item details.

szPMNull	db	0
		align	dword

; Menu strings.

szBytes		db	" bytes  ",0
		align	dword
szTimes		db	" x ",0
		align	dword
szPixels	db	" pixels  ",0
		align	dword
szGroup		db	"Group: ",0
		align	dword
szSearch	db	"    Contains text: ",0
		align	dword
szYes		db	"Yes",0
		align	dword
szNo		db	"No",0
		align	dword
szType		db	"    Item Type: ",0
		align	dword

szPMGoto	db	"Explore Item location",0
		align	dword

szExplore	db	"explore",0		; (for ShellExecute)
		align	dword

szPMEdit	db	"Edit Item Properties",0
		align	dword

szPMExit	db	"Close this Menu",0
		align	dword




; Color initialisation.

szTMIS_COLR	db	"Color",0		; Section name

szTMIK_TWBG	db	"TimeWindowBG",0	; Key names
szTMIK_ITBG	db	"ItemBG",0
szTMIK_TSTX	db	"ItemTSText",0
szTMIK_TBBG	db	"TimeBoxBG",0




; Object allocation.

TM_OAF		dd	0

; Flags, bit set indicates object/resource etc are allocated.
; These are used in the deallocation routine.

TM_OAF_Colors	equ	1 shl 0		; colors loaded




; Time module error handling.


; Time module error codes.

ERR_TM_Module	equ	1300h

ERR_TMInit	equ	ERR_TM_Module + 0	; (placeholder only)
ERR_ImageLoad	equ	ERR_TM_Module + 1
ERR_WndReg	equ	ERR_TM_Module + 2
ERR_WndCreate	equ	ERR_TM_Module + 3


; Time_Init error messages.
;
; sERR_TMInit has one of the following error messages appended.

sERR_TMInit	db	"Time Window Init failed - ",0
sERR_ImageLoad	db	"Up/Dn bitmap load",0
sERR_WndReg	db	"Window registration",0
sERR_WndCreate	db	"Window create",0
		align	dword

ErrorMsgs	dd	offset sERR_TMInit
		dd	offset sERR_ImageLoad
		dd	offset sERR_WndReg
		dd	offset sERR_WndCreate

nErrors		equ	4




.code




; Public functions.

include		timepub.asm




; Private functions.

option		proc:private




if ShowTDisp	;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		include	timedbg.asm
endif		;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++




include		timeshow.asm




;----------------------------------------------------------------------------
TimeWndHdlr	proc	uses ebx edi esi,
			hWnd:dword, uMsg:dword, wParam:dword, lParam:dword
;----------
;
; Time Window message handler.
;
; Messages ordered most used to least used for speed.
;
; Note the Time window position in the Z order, activation etc is handled
; in the SideBar module as it has to keep track of itself and its dialogs
; as well as the Time window.
;
; Note there is no easy way to wait for ShellExecute to return and then put
; up the Time window afterward - this is because it doesn't return a valid
; process ID for OpenProcess to return a handle that can be used for a
; WaitForSingleObject poll.
;----------------------------------------------------------------------------


	local	ps			:PAINTSTRUCT
	local	szIPath[MAX_PATH]	:byte		; full Item path
	local	y			:dword		; y coordinate
	local	szNumber[12]		:byte		; conversion string


	mov	eax, uMsg

	cmp	eax, WM_MOUSEMOVE
	jz	@@OnMouse
	cmp	eax, WM_TIMER
	jz	@@OnTimer
	cmp	eax, WM_MOUSEWHEEL
	jz	@@OnWheel
	cmp	eax, WM_PAINT
	jz	@@OnPaint
	cmp	eax, WM_KEYDOWN
	jz	@@OnKeyDown
	cmp	eax, WM_LBUTTONDOWN
	jz	@@OnLMBDown
	cmp	eax, WM_LBUTTONUP
	jz	@@OnLMBUp
	cmp	eax, WM_LBUTTONDBLCLK
	jz	@@OnLMBDblClk
	cmp	eax, WM_CONTEXTMENU
	jz	@@OnContxtMenu
	cmp	eax, WM_COMMAND
	jz	@@OnCommand
	cmp	eax, WM_ACTIVATE
	jz	@@OnActivate
	cmp	eax, WM_CLOSE
	jz	@@OnClose
	cmp	eax, WM_DESTROY
	jz	@@OnDestroy
	cmp	eax, WM_CREATE
	jz	@@OnCreate

	jmp	@@DefMsg


@@OnMouse:	;------------------------------------------------------------
		; Mouse cursor moved.


	; If movement disabled then ignore this message.

	.if	(bTimeStop)
		jmp	@@MsgHandled
	.endif

	movzx	eax, lPHi			; mouse y position
	mov	ecx, G_ScrnHgt
	dec	ecx				; (bottom line)

	.if	(eax == 0)

		; If mouse touching window top edge,

		mov	bMouseTop, 1
		fld	zAdj			; then move backward in time,
		fsub	zM			; so z is decremented.
		fstp	zAdj
		invoke	ShowTime

	.elseif	(eax == ecx)

		; Else if mouse touching window bottom edge,

		mov	bMouseBot, 1
		fld	zAdj			; then move forward in time,
		fadd	zM			; so z is incremented.
		fstp	zAdj
		invoke	ShowTime

	.else

		; Else if mouse moving Move Time box then move it.

		.if	(bMovTimeMov)

			; Index = ((Y - y)/Y) * Items
			;
			; This is the reverse of the algorithm in DrawTime.

			mov	edx, TMRange	; Y
			sub	edx, eax	; Y - y
			mov	y, edx
			fild	y
			fidiv	TMRange		; / Y
			fimul	SrchItems	; * Items
			fistp	y		; = index
			mov	eax, y

			; Initialise the new BaseItem but keep it
			; in range 0 to (SrchItems - DispItems).

			mov	edx, eax
			add	edx, DispItems

			.if	(sdword ptr eax < 0)

				xor	eax, eax
				mov	edx, eax

			.elseif	(edx >= SrchItems)

				mov	edx, SrchItems
				sub	edx, DispItems

			.else

				mov	edx, eax

			.endif

			mov	BaseItem, edx

			; Set the z adjustment.

			dec	eax
			mov	y, eax
			fild	y
			fchs			; (zAdj is negative)
			fstp	zAdj

			; Show the movement.

			invoke	ShowTime

		.endif

		; Else neither edge.

		xor	eax, eax
		mov	bMouseTop, eax		; reset flags
		mov	bMouseBot, eax

	.endif

	jmp	@@MsgHandled


@@OnTimer:	;------------------------------------------------------------
		; Auto Move timer.


	; If movement disabled then ignore this message.

	.if	(bTimeStop)
		jmp	@@MsgHandled
	.endif

	mov	eax, wParam

	.if	(eax == AutoMoveTimer)		; Ensure it is our timer

		mov	eax, bMouseTop

		.if	(eax)			; If mouse still at top,

			fld	zAdj
			fsub	zT		; keep moving backward.
			fstp	zAdj
			invoke	ShowTime

			jmp	@@MsgHandled

		.endif

		mov	eax, bMouseBot

		.if	(eax)			; If mouse still at bottom,

			fld	zAdj
			fadd	zT		; keep moving forward.
			fstp	zAdj
			invoke	ShowTime

			jmp	@@MsgHandled

		.endif

		jmp	@@MsgHandled

	.else

		jmp	@@DefMsg

	.endif


@@OnWheel:	;------------------------------------------------------------
		; Clitoris caress.


	mov	eax, wParam
	sar	eax, 16				; wheel delta (keep sign bit)

	.if	(sdword ptr eax > 0)		; Wheel rotated forward
						; (away from user)
		fld	zAdj			; moves forward in time,
		fadd	zW			; so z is incremented.
		fstp	zAdj

	.else					; Wheel rotated backward
						; (toward user)
		fld	zAdj			; moves backward in time,
		fsub	zW			; so z is decremented.
		fstp	zAdj

	.endif

	invoke	ShowTime

	jmp	@@MsgHandled


@@OnPaint:	;------------------------------------------------------------


	invoke	BeginPaint, hWnd, addr ps	; Must tell system,
	invoke	ShowTime
	invoke	EndPaint, hWnd, addr ps		;  we did it.

	jmp	@@MsgHandled


@@OnKeyDown:	;------------------------------------------------------------
		; Keypress.
		;
		; Handle all keypresses but action only Up/Down.


	; Alter Auto Move speed.

	movzx	eax, wPLo

	.if	(eax == VK_UP)

		fld	zT
		fadd	zDT			; Move faster
		fstp	zT

	.elseif	(eax == VK_DOWN)

		fld	zT
		fsub	zDT			; Move slower
		fstp	zT

	.endif

	jmp	@@MsgHandled


@@OnLMBDown:	;------------------------------------------------------------
		; Mouse Left button down.


	movzx	eax, lPLo			; Mouse x
	movzx	edx, lPHi			; Mouse y


	; Scan for mouse inside Move Time box.
	;
	; If so then turn on Move box movement to enable OnMouse to do it.
	;
	; We do Move box first so if it is over the Start or End boxes
	; then we can pick it up and move it.

	cmp	eax, MovTime.left
	jb	@@NotInMov

	cmp	eax, MovTime.right
	ja	@@NotInMov

	cmp	edx, MovTime.top
	jb	@@NotInMov

	cmp	edx, MovTime.bottom
	ja	@@NotInMov

	mov	bMovTimeMov, 1

	jmp	@@MsgHandled

	@@NotInMov:

	mov	bMovTimeMov, 0


	; Scan for mouse inside Start Time box.
	; If so then set the time display to the start.

	cmp	eax, StaTime.left
	jb	@@NotInSta

	cmp	eax, StaTime.right
	ja	@@NotInSta

	cmp	edx, StaTime.top
	jb	@@NotInSta

	cmp	edx, StaTime.bottom
	ja	@@NotInSta

	invoke	Time_SetItem, TSetFirst, 0, SrchItems
	invoke	ShowTime

	jmp	@@MsgHandled

	@@NotInSta:


	; Scan for mouse inside End Time box.
	; If so then set the time display to the end.

	cmp	eax, EndTime.left
	jb	@@NotInEnd

	cmp	eax, EndTime.right
	ja	@@NotInEnd

	cmp	edx, EndTime.top
	jb	@@NotInEnd

	cmp	edx, EndTime.bottom
	ja	@@NotInEnd

	invoke	Time_SetItem, TSetLast, 0, 0
	invoke	ShowTime

	jmp	@@MsgHandled

	@@NotInEnd:


	; Now check if we are in MoveTime up/down icons.

	cmp	edx, MovTime.top
	jb	@@MsgHandled

	cmp	edx, MovTime.bottom
	ja	@@MsgHandled

	mov	ecx, MovTime.right
	add	ecx, MoveIcoXY			; icon width
	cmp	eax, ecx
	ja	@@QryInUp


	; In Down icon, reset Search array to Current item as new
	; Start time.

	invoke	Time_SetItem, TSetFirst, CurrItem, SrchItems
	invoke	ShowTime

	jmp	@@MsgHandled


	@@QryInUp:

	add	ecx, MoveIcoXY			; icon width
	cmp	eax, ecx
	ja	@@MsgHandled


	; In Up icon, reset Search array to Current item as new End time.

	mov	eax, CurrItem
	inc	eax				; Items in Search array
	invoke	Time_SetItem, TSetLast, 0, eax
	invoke	ShowTime

	jmp	@@MsgHandled


@@OnLMBUp:	;------------------------------------------------------------
		; Mouse Left button up.


	mov	bMovTimeMov, 0			; reset always

	jmp	@@MsgHandled


@@OnLMBDblClk:	;------------------------------------------------------------
		; Mouse Left button double click.


	movzx	eax, lPLo			; Mouse x
	movzx	edx, lPHi			; Mouse y

	; Scan for mouse inside Large Item rectangle.

	invoke	QryInLargeItem

	.if	(CARRY?)
		jmp	@@MsgHandled		; no
	.endif

	; Double-click in Large Item, open it.
	;
	; First get Item's full path.

	lea	edx, szIPath
	invoke	DB_GetNfo, I_GETFULL, LI[eax].Ndx, edx ;addr szIPath

	; Minimise Time window and stop, placing the Desktop foremost.

	invoke	SBar_DeskUp

	; Execute "open" for Item.

	invoke	ShellExecuteA, hWnd, NULL, addr szIPath, NULL, NULL,
			SW_SHOWNORMAL

	jmp	@@MsgHandled


@@OnContxtMenu:	;------------------------------------------------------------
		; Mouse Right button click.


	movzx	eax, lPLo			; Mouse x
	movzx	edx, lPHi			; Mouse y


	; Show Context Menu.


	; Scan for mouse inside Large Item rectangle.

	invoke	QryInLargeItem

	.if	(CARRY?)
		jmp	@@MsgHandled		; no
	.endif


	; Load Item's full path.

	mov	ebx, LI[eax].Ndx		; Item

	mov	MenuedItem, ebx			; save Item

	invoke	DB_GetNfo, I_GETFULL, ebx, addr szIPath

	invoke	File_Exist, addr szIPath

	.if	(CARRY?)
		jmp	@@MsgHandled		; file does not exist
	.endif

	push	eax				; file size

	invoke	ModifyMenu, hPupMenu, IDM_PATH, MF_BYCOMMAND or MF_STRING,
			IDM_PATH, addr szIPath


	; Load Item's size and if an image, its dimensions.

	pop	edx				; file size

	invoke	dwtoa, edx, addr szIPath
	invoke	szCatStr, addr szIPath, addr szBytes

	invoke	DB_GetNfo, I_GETTHUM, ebx, NULL

	.if	((eax == FTH_Img) && G_GdipOk)

		mov	edx, ebx
		mov	ecx, ebx
		and	ecx, nImgObj-1		; handle 0 up
		shl	ecx, ImgObjRecMult	; * bytes per object
		cmp	edx, ImgObj[ecx].Item
		jnz	@@ImgNotLoaded

		mov	eax, ImgObj[ecx].ImgXY
		mov	edx, eax
		and	eax, 0FFFFh		; y

		push	eax			; save y

		shr	edx, 16			; x
		invoke	dwtoa, edx, addr szNumber
		invoke	szCatStr, addr szIPath, addr szNumber
		invoke	szCatStr, addr szIPath, addr szTimes

		pop	edx			; y

		invoke	dwtoa, edx, addr szNumber
		invoke	szCatStr, addr szIPath, addr szNumber
		invoke	szCatStr, addr szIPath, addr szPixels

	.endif

	@@ImgNotLoaded:

	invoke	ModifyMenu, hPupMenu, IDM_SIZE, MF_BYCOMMAND or MF_STRING,
			IDM_SIZE, addr szIPath


	; Show Item file specification.

	invoke	szCopy, addr szGroup, addr szIPath
	invoke	DB_GetNfo, I_GETGRPS, ebx, NULL
	mov	edx, eax
	invoke	szCatStr, addr szIPath, edx

	invoke	DB_GetNfo, I_GETGRPI, ebx, NULL

	.if	(eax > kEditGroups)

		; Cannot edit these ones.

		invoke	EnableMenuItem, hPupMenu, IDM_EDIT,
				MF_DISABLED or MF_GRAYED

	.else

		; Can edit these ones.

		invoke	szCatStr, addr szIPath, addr szSearch
		invoke	DB_GetNfo, I_GETSRCH, ebx, NULL
		.if	(eax)
			mov	edx, offset szYes
		.else
			mov	edx, offset szNo
		.endif
		invoke	szCatStr, addr szIPath, edx

	.endif

	invoke	ModifyMenu, hPupMenu, IDM_SPEC, MF_BYCOMMAND or MF_STRING,
			IDM_SPEC, addr szIPath


	; Put up the Menu.
	;
	; The TimeWndHdlr handles the Menu messages of which WM_COMMAND is
	; sent when the Menu is closed.

	movzx	eax, lPLo			; Mouse x
	movzx	edx, lPHi			; Mouse y

	invoke	TrackPopupMenu, hPupMenu, TPM_LEFTALIGN, eax, edx, 0, hWnd, 0


	; Re-enable Edit, jic disabled above.

	invoke	EnableMenuItem, hPupMenu, IDM_EDIT, MF_ENABLED


	jmp	@@MsgHandled


@@OnCommand:	;------------------------------------------------------------
		; Mouse click on buttons/controls.


	; Click in Popup Menu?.

	.if	(lParam == 0)

		; Message is from the Menu.

		movzx	eax, wPLo		; Menu item identifier

		.if	(eax == IDM_GOTO)


			; Explore Item Location.

			; Get Item's path (trailing '\' is ok).

			mov	ebx, MenuedItem
			invoke	DB_GetNfo, I_GETPATH, ebx, NULL
			push	eax

			; Minimise Time window, placing the Desktop foremost.

			invoke	SBar_DeskUp

			; Execute "explore" for Item.

			pop	edx
			invoke	ShellExecuteA, hWnd, addr szExplore, edx,
					NULL, NULL, SW_SHOWMAXIMIZED


		.elseif	(eax == IDM_EDIT)


			; Edit Item Properties.

			invoke	DialogBoxParamA, G_hTui, ID_IP_DLG, hTimeWnd,
					addr EditDlgHdlr, NULL


		.endif

	.endif

	jmp	@@MsgHandled


@@OnActivate:	;------------------------------------------------------------
		; Activate/DeActivate.


	movzx	eax, wPLo			; Activation flag

	.if	(eax == WA_INACTIVE)


		; Whenever Time window is inactivated, disable automove timer
		; and mouse movement.

		mov	bTimeStop, TRUE

		; Time window inactive, trigger any pending database update.

		invoke	SetEvent, G_UDBhEvent


		if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_TM_WMAct, FALSE, lParam
		endif		;++++++++++++++++++++++++++++++++++++++++++++


	.else

		; Time window active, disable database update.
		;
		; Reset the Event trigger jic it hasn't been triggered yet,
		; as would be the case if the event has been signalled but
		; there is a delay due to a scan of the drives on the system.

		invoke	ResetEvent, G_UDBhEvent

		; Spin here until a database update is finished.

		invoke	WaitForSingleObject, G_UDBhMutex, INFINITE
		invoke	ReleaseMutex, G_UDBhMutex

		; Whenever Time window is activated, enable automove timer
		; and mouse movement.

		mov	bTimeStop, FALSE


		if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_TM_WMAct, TRUE, lParam
		endif		;++++++++++++++++++++++++++++++++++++++++++++


	.endif

	jmp	@@MsgHandled


@@OnClose:	;------------------------------------------------------------
		; Close window, Alt-F4 etc.


	; Return as handled, thus Alt-F4 is disabled.

	jmp	@@MsgHandled


@@OnDestroy:	;------------------------------------------------------------
		; Destroy the Time Window.


	; Auto Move Timer off.

	invoke	KillTimer, hWnd, AutoMoveTimer

	; Release objects in reverse order to creation.

	invoke	DeleteObject, hFontB
	invoke	DeleteObject, hFont
	invoke	DeleteObject, hItemBrush
	invoke	DeleteObject, hWhiteBrush
	invoke	DeleteObject, hBGBrush
	invoke	DeleteObject, hBitmap

	invoke	DestroyMenu, hPupMenu

	.if	(G_GdipOk)

		; Release all GDI+ objects.

		invoke	GdipDeleteGraphics, hGdipGraObj

		mov	ecx, nImgObj

		.repeat

			dec	ecx
			push	ecx
			shl	ecx, ImgObjRecMult
			push	ecx
			invoke	GdipDisposeImage, ImgObj[ecx].hBmp
			pop	ecx
			invoke	GdipDeleteGraphics, ImgObj[ecx].hGra
			pop	ecx

		.until	(ecx == 0)

	.endif

	invoke	DeleteDC, hMemDC
	invoke	ReleaseDC, hWnd, hDC

	jmp	@@MsgHandled


@@OnCreate:	;------------------------------------------------------------
		; Create the Time Window.
		;
		; Much quicker to create our objects here (2-3x times faster)
		; than in the Draw function. Objects are released at
		; WM_DESTROY in the reverse order they were created.


	; Get client rectangle and create a compatible DC and bitmap
	; to use as the back buffer.

	invoke	GetClientRect, hWnd, addr ScrnRect

	invoke	GetDC, hWnd
	mov	hDC, eax

	invoke	CreateCompatibleDC, hDC
	mov	hMemDC, eax

	mov	eax, ScrnRect.right
	sub	eax, ScrnRect.left
	mov	edx, ScrnRect.bottom
	sub	edx, ScrnRect.top
	invoke	CreateCompatibleBitmap, hDC, eax, edx
	mov	hBitmap, eax


	; Create right-click popup menu.

	invoke	CreatePopupMenu
	mov	hPupMenu, eax

	invoke	AppendMenu, hPupMenu, MF_STRING, IDM_PATH, addr szPMNull
	invoke	AppendMenu, hPupMenu, MF_STRING, IDM_SIZE, addr szPMNull
	invoke	AppendMenu, hPupMenu, MF_STRING, IDM_SPEC, addr szPMNull
	invoke	AppendMenu, hPupMenu, MF_SEPARATOR, 0, 0
	invoke	AppendMenu, hPupMenu, MF_STRING, IDM_GOTO, addr szPMGoto
	invoke	AppendMenu, hPupMenu, MF_STRING, IDM_EDIT, addr szPMEdit
	invoke	AppendMenu, hPupMenu, MF_SEPARATOR, 0, 0
	invoke	AppendMenu, hPupMenu, MF_STRING, IDM_EXIT, addr szPMExit


	; GDI+, must select bitmap and create graphics object for it.

	.if	(G_GdipOk)

		invoke	SelectObject, hMemDC, hBitmap
		push	eax
		invoke	GdipCreateFromHDC, hMemDC, addr hGdipGraObj
		pop	eax
		invoke	SelectObject, hMemDC, eax	; replace original

		; Default and 'low quality' mode appears to be
		; InterpolationModeBilinear. Nearest neighbour
		; works ok.

		invoke	GdipSetInterpolationMode, hGdipGraObj,
				InterpolationModeNearestNeighbor

		mov	ecx, nImgObj

		.repeat

			dec	ecx
			push	ecx
			shl	ecx, ImgObjRecMult
			push	ecx

			; Create new bitmap of Thumbnail width and height and
			; same pixel format as the original image.
			; Bitmap::Bitmap(width, height, format (see headers)

			invoke	GdipCreateBitmapFromScan0, ItemThmWid,
					ItemThmHgt, 0, PixelFormat32bppARGB,
					NULL, addr ImgObj[ecx].hBmp

			; Create new graphics object from new image.

			pop	ecx
			invoke	GdipGetImageGraphicsContext, ImgObj[ecx].hBmp,
					addr ImgObj[ecx].hGra
			pop	ecx

		.until	(ecx == 0)

	.endif


	; Brushes.

	invoke	CreateSolidBrush, CTimeWnd
	mov	hBGBrush, eax

	invoke	CreateSolidBrush, CWhite
	mov	hWhiteBrush, eax

	invoke	CreateSolidBrush, CTimeItem
	mov	hItemBrush, eax


	; We want the font height in a Time box rectangle so we
	; need to select it, DrawText to return the rectangle height
	; and then replace the original font.
	;
	; Also we want the maximum width for our boxes so the DrawText
	; call uses MovTime with dummy widest char initialisation which
	; is in initialised data MoveTime structure.

	invoke	CreateFontIndirect, addr FontSpec
	mov	hFont, eax

	invoke	SelectObject, hDC, eax

	push	eax

	; DrawText with DT_CALCRECT sets the width in MovTime.right
	; and returns the text height.

	invoke	DrawTextA, hDC, addr MovTime.TSStrg, -1, addr MovTime,
			DT_CALCRECT

	invoke	SetTimeParams, eax

	pop	eax

	invoke	SelectObject, hDC, eax		; replace original


	; Set bold font.

	mov	FontSpec.lfWeight, FW_SEMIBOLD
	invoke	CreateFontIndirect, addr FontSpec
	mov	hFontB, eax


	; Initialise Auto Move Timer.

	invoke	SetTimer, hWnd, AutoMoveTimer, MoveTmrPeriod, NULL


	; and fall into MsgHandled...


@@MsgHandled:	;------------------------------------------------------------


	; Inform the system we have handled the message.

	xor	eax, eax


	ret


@@DefMsg:	;------------------------------------------------------------


	invoke	DefWindowProcA, hWnd, uMsg, wParam, lParam


	ret


TimeWndHdlr	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
SetTimeParams	proc		TxtHgt:dword
;------------
;
; Setup Time window parameters.
;
; Convenience function to keep Time window WM_CREATE less unwieldy so ONLY
; to be called from there.
;
;
; Entry		TxtHgt		height of text in Time boxen
;
;
; Exit		various parameters initialised
;----------------------------------------------------------------------------


; Screen Width and Height / 2.


	mov	eax, G_ScrnWid
	shr	eax, 1
	mov	xMid, eax
	mov	eax, G_ScrnHgt
	shr	eax, 1
	mov	yMid, eax


; Set the height and right edge of boxes.


	mov	eax, TxtHgt
	mov	TBoxHgt, eax			; Time box height
	mov	edx, G_ScrnHgt			; Y pixels
	sub	edx, eax			; - Box height
	mov	TMRange, edx			; = Time Move range
	dec	eax
	mov	TBoxHgt_1, eax			; convenient

	mov	eax, MovTime.right
	mov	StaTime.right, eax
	mov	EndTime.right, eax


; Set the box positions.
;
; End Time box		top left of screen
; Start Time box	bottom left of screen
; Move Time box		dynamically positioned on left edge
; ItemsBox box		center bottom of screen


	mov	eax, G_ScrnHgt
	dec	eax				; pixels # 0 -> G_ScrnHgt-1
	mov	StaTime.bottom, eax
	mov	ItemsBox.bottom, eax
	sub	eax, TBoxHgt_1
	mov	StaTime.top, eax
	mov	ItemsBox.top, eax

	xor	eax, eax
	mov	EndTime.top, eax
	add	eax, TBoxHgt_1
	mov	EndTime.bottom, eax

	mov	eax, xMid
	sub	eax, 50				; (arbitrary centering!)
	mov	ItemsBox.left, eax
	add	eax, 200			; (sized by DrawTextA)
	mov	ItemsBox.right, eax


	ret


SetTimeParams	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
QryInLargeItem	proc	uses ebx
;-------------
;
; Scan for mouse cursor inside any Large Item rectangle.
;
;
; Entry		eax		Mouse x
;		edx		Mouse y
;
;		External:
;
;		LINdx		current lowest index
;		LI		Large Item array
;
;
; Exit		carry	clear	inside Large Item
;				eax has	LI array offset for Item
;
;		carry	set	not in any Large Item
;----------------------------------------------------------------------------


	mov	ecx, LINdx			; current lowest index

	.repeat

		lea	ebx, [ecx + ecx*4]	; Item * 5
		shl	ebx, 2			;  * 4 = sizeof LIstruc

		cmp	eax, LI[ebx].Coord.left
		jb	@@NotInItem

		cmp	eax, LI[ebx].Coord.right
		ja	@@NotInItem

		cmp	edx, LI[ebx].Coord.top
		jb	@@NotInItem

		cmp	edx, LI[ebx].Coord.bottom
		ja	@@NotInItem

		jmp	@@EndScan

		@@NotInItem:
		inc	ecx

	.until	(ecx == nLgItems)

	@@EndScan:

	.if	(ecx == nLgItems)
		stc
	.else
		clc
		mov	eax, ebx
	.endif


	ret


QryInLargeItem	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
EditDlgHdlr	proc	uses ebx esi edi,
			hWnd:dword,uMsg:dword,wParam:dword,lParam :dword
;----------
;
; Item Edit Properties Dialog message handler.
;
;
; Exit		!0	Message processed
;
;		0	Message not processed - the dialog manager performs
;			the default dialog operation for the message.
;
;		End	NULL
;----------------------------------------------------------------------------


	local	szBuf[MAX_PATH]		:byte		; string buffer
	local	szSfx[8]		:byte		; suffix buffer


	mov	eax, uMsg

	cmp	eax, WM_INITDIALOG
	jz	@@OnInit
	cmp	eax, WM_COMMAND
	jz	@@OnCommand
	cmp	eax, WM_ACTIVATE
	jz	@@OnActivate
	cmp	eax, WM_CLOSE
	jz	@@OnClose

	xor	eax, eax			; Msg not processed
	jmp	@@ExitHandler


@@OnInit:	;------------------------------------------------------------
		; Initialise the Dialog box.


	; Get our Item.

	mov	ebx, MenuedItem


	; Load the File name.

	invoke	DB_GetNfo, I_GETNAME, ebx, NULL
	invoke	SendDlgItemMessage, hWnd, ID_IP_TXT1, WM_SETTEXT, 0, eax


	; Load the File Properties.

	invoke	szCopy, addr szGroup, addr szBuf
	invoke	DB_GetNfo, I_GETGRPS, ebx, NULL
	mov	edx, eax
	invoke	szCatStr, addr szBuf, edx

	invoke	szCatStr, addr szBuf, addr szSearch
	invoke	DB_GetNfo, I_GETSRCH, ebx, NULL
	.if	(eax)
		mov	edx, offset szYes
	.else
		mov	edx, offset szNo
	.endif
	invoke	szCatStr, addr szBuf, edx

	invoke	szCatStr, addr szBuf, addr szType

	invoke	DB_GetNfo, I_GETFSFX, ebx, addr szSfx
	invoke	szCatStr, addr szBuf, addr szSfx

	invoke	SendDlgItemMessage, hWnd, ID_IP_TXT2, WM_SETTEXT, 0,
			addr szBuf


	; Load the ComboBox with the editable Group names.

	push	ebx

	xor	ebx, ebx
	inc	ebx

	.repeat

		invoke	DB_GetGrpName, ebx

		invoke	SendDlgItemMessage, hWnd, ID_IP_CBGRP,
				CB_ADDSTRING, 0, eax
		inc	ebx

	.until	(ebx > kEditGroups)


	; Set the Item's Group to be selected when the box is shown.

	pop	ebx

	invoke	DB_GetNfo, I_GETGRPI, ebx, NULL
	dec	eax				; 0 based

	invoke	SendDlgItemMessage, hWnd, ID_IP_CBGRP, CB_SETCURSEL, eax, 0


	; Set the state of the checkbox according to the Item's searchable
	; flag.

	invoke	DB_GetNfo, I_GETSRCH, ebx, NULL

	.if	(eax)
		mov	eax, BST_CHECKED
	.else
		mov	eax, BST_UNCHECKED
	.endif

	invoke	CheckDlgButton, hWnd, ID_IP_CKTXT, eax

	jmp	@@MsgHandled


@@OnCommand:	;------------------------------------------------------------
		; Mouse click on buttons/controls.


	movzx	eax, wPLo

	.if	(eax == IDOK)


		; Get Group selected and state of 'Contains Text' checkbox.

		invoke	SendDlgItemMessage, hWnd, ID_IP_CBGRP,
				CB_GETCURSEL, 0, 0
		mov	ebx, eax

		invoke	IsDlgButtonChecked, hWnd, ID_IP_CKTXT

		xor	edx, edx

		.if	(eax == BST_CHECKED)
			inc	edx
		.endif


		; Set the type and update the display.

		invoke	DB_SetTypeSpec, MenuedItem, ebx, edx
		invoke	ShowTime

		jmp	@@CloseDialog


	.elseif	(eax == IDCANCEL)


		@@CloseDialog:

		invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL


	.endif

	jmp	@@MsgHandled


@@OnActivate:	;------------------------------------------------------------
		; Activate/DeActivate (see SBAR.ASM, window focus notes).


	; This will close the Menu if the user mouses into the right side of
	; the screen, thus opening the SideBar. Note that clicking outside
	; the Menu has no action.

	movzx	eax, wPLo			; Activation flag

	.if	(eax == WA_INACTIVE)

		invoke	PostMessageA, hWnd, WM_CLOSE, NULL, NULL

	.endif

	jmp	@@MsgHandled


@@OnClose:	;------------------------------------------------------------
		; Close Dialog box, Alt-F4 etc.


	invoke	EndDialog, hWnd, 0


@@MsgHandled:	;------------------------------------------------------------
		; Exit, Dialog message handled.


	mov	eax, TRUE


@@ExitHandler:


	ret


EditDlgHdlr	endp
;----------------------------------------------------------------------------




end	; Time module.




;============================================================================
