;============================================================================
; TUI.INC
;
; Systemwide definitions, included in every module.
;============================================================================




; Model ---------------------------------------------------------------------


.586p
.model		flat, stdcall		; MS Windows
option		casemap :none		; case sensitive




; Debug +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;
; Flags are 0 for false, 1 for true.
;
; All mainline debug code has ";++++.. " as delimiters.


; VKDebug, LogFile
; ----------------


; It seems advisable to pushad/popad around VKDebug functions, especially in
; message handlers as sometimes windows will behave oddly. Where the VKDebug
; window interferes with messaging (WM_ACTIVATE especially), use the log file.
; Note the log file creation will cause a database update trigger.

Dbg_VK		equ	0

; Dbg_Log includes \debug\dbglog.*

Dbg_Log		equ	0


; Tui messaging
; -------------


; Log window messaging and threading.
;
; Dbg_Msgs includes \debug\dbgmsgs.*
;
; (Better not to have any VKDebug windows happening)

Dbg_Msgs	equ	0 AND Dbg_Log AND NOT Dbg_VK


; GDI+
; ----


; Do not load or use GDI+ subsystem.
;
; No image based thumbnailing can be done.

GdipOff		equ	0


; Database Module
; ---------------


; Database built from drive C only.
;
; Used in UpdateDB, restricts database size for testing.

HDC_Only	equ	0
;
; AND if for D instead of C (my D drive has very few files):
;
HDD_SubstD	equ	0 AND HDC_Only


; Time Module
; -----------


; Debugging Time display:
;
; * Reset DefDisplayItems, altered in time.asm
; * Sets Wheel increments smallest
; * Enables DrawTime and DrawItem debug display
; * Requires VKDebug

ShowTDisp	equ	0 AND Dbg_VK

; Show timing for DrawTime.

ShowDTTime	equ	0 AND Dbg_VK




; Masm32 --------------------------------------------------------------------
;
; Includelib lines are passed to linker (Philippe Auphelle's win32asm.txt).


.nolist

include		\masm32\include\windows.inc

include		\masm32\include\comctl32.inc
include		\masm32\include\comdlg32.inc
include		\masm32\include\gdi32.inc
include		\masm32\include\kernel32.inc
include		\masm32\include\masm32.inc
include		\masm32\include\shell32.inc
include		\masm32\include\user32.inc

includelib	\masm32\lib\comctl32.lib
includelib	\masm32\lib\comdlg32.lib
includelib	\masm32\lib\gdi32.lib
includelib	\masm32\lib\kernel32.lib
includelib	\masm32\lib\masm32.lib
includelib	\masm32\lib\shell32.lib
includelib	\masm32\lib\user32.lib

include		\masm32\macros\macros.asm


;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Debug stuff.

if Dbg_VK
		include		\masm32\include\debug.inc
		includelib	\masm32\lib\debug.lib
		include		\masm32\macros\timers.asm
endif

if Dbg_Log
		include		debug\dbglog.inc
endif

if Dbg_Msgs
		include		debug\dbgmsgs.inc
endif

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

.listall




; General Constants ---------------------------------------------------------


; Equates for splitting the wParam and lParam passed to a WndProc into their
; word size components. They take the word value directly off the stack
; without any conversions. Sign extend to dword with "movsx <reg>, wpLow" etc.
; From Ernie Murphy.

wPLo		equ	<word ptr [ebp + 16]>
wPHi		equ	<word ptr [ebp + 18]>
lPLo		equ	<word ptr [ebp + 20]>
lPHi		equ	<word ptr [ebp + 22]>


; Text output.

TB		equ	09h		; tab
LF		equ	0Ah
CR		equ	0Dh


; Colors: 0x00BBGGRR (api COLORREF format).

CWhite		equ	00FFFFFFh
CBlue		equ	00FF0000h
CYellow		equ	0000FFFFh
CRed		equ	000000FFh
CBlack		equ	00000000h


; SideBar.

SbarWidth	equ	26		; pixels




; Tui Errorcodes ------------------------------------------------------------


; These give the range of error codes - see Modules and TuiFatalError.
;
; Tui/Common	1000h - 10FFh
; Database	1100h - 11FFh
; SideBar	1200h - 12FFh
; Time		1300h - 13FFh

TuiErrMin	equ	1000h
TuiErrMax	equ	13FFh




; Resources -----------------------------------------------------------------
;
; See TUI.RC


; Tui Icon.
;
; Make this the first so it becomes the app icon by default.

ID_IC_TUI	equ	0000		; (never referenced)


; Group Icons (== File Group IDs, must be 1 up).
;
; Put the non-assignable ones at the end of the list (see below).
; kEditGroups sets the last Group that allows Properties editing in
; the Time Window Item context menu. It disables Groups that should
; not be assigned by a user.
;
; Note that all the Groups can be selected to be displayed or not.

FG_Unk		equ	1		; unknown
FG_Wrd		equ	2		; words
FG_Num		equ	3		; numbers
FG_Snd		equ	4		; sounds
FG_Vid		equ	5		; movies
FG_Net		equ	6		; net
FG_Hlp		equ	7		; help
FG_Ark		equ	8		; archives
FG_Exe		equ	9		; tools
FG_Sys		equ	10		; system
FG_Src		equ	11		; sources

kEditGroups	equ	11

FG_Pix		equ	12		; pictures (reserved)
FG_Lnk		equ	13		; shortcuts
FG_Del		equ	14		; recycled

kGroups		equ	14		; total icons/groups


; Time window icons.

ID_TM_UPICON	equ	1000
ID_TM_DNICON	equ	1001


; Item Properties Dialog.

ID_IP_DLG	equ	1100
ID_IP_GRP1	equ	1101
ID_IP_TXT1	equ	1102
ID_IP_TXT2	equ	1103
ID_IP_TXT3	equ	1104
ID_IP_CBGRP	equ	1105
ID_IP_CKTXT	equ	1106


; New Database Dialog.

ID_ND_DLG	equ	2000
ID_ND_PGB1	equ	2010


; SideBar ToolBar.

ID_SB_TB	equ	4100		; ToolBar ID
ID_SB_TBBMP	equ	4101		; ToolBar bitmap ID

SB_BMP_TIME	equ	0		; ToolBar button bitmap
SB_BMP_DESK	equ	1		;   order in SBAR.BMP
; Separator
SB_BMP_FIND	equ	2
SB_BMP_GRP	equ	3
SB_BMP_THUM	equ	4
; Separator
SB_BMP_EXPT	equ	5
SB_BMP_OPTN	equ	6
SB_BMP_HELP	equ	7
; Separator
SB_BMP_EXIT	equ	8

kTBButtons	equ	9 + 3		; Must count separators

ID_SB_BTTIME	equ	4110		; Command identifiers for
ID_SB_BTDESK	equ	4111		; ToolBar buttons.
ID_SB_BTFIND	equ	4112
ID_SB_BTGRP	equ	4113
ID_SB_BTTHUM	equ	4114
ID_SB_BTEXPT	equ	4115
ID_SB_BTOPTN	equ	4116
ID_SB_BTHELP	equ	4117
ID_SB_BTEXIT	equ	4118


; Find Dialog.

ID_FN_DLG	equ	4200
ID_FN_EDIT	equ	4201
ID_FN_CKCASE	equ	4202
ID_FN_CKNEND	equ	4203
ID_FN_RDNAME	equ	4204
ID_FN_RDDATA	equ	4205
ID_FN_RDPATH	equ	4206


; Group Dialog.

ID_GR_DLG	equ	4300
ID_GR_LSVGRP	equ	4301


; Export Dialog.

ID_EX_DLG	equ	4500


; Options Dialog.

ID_OP_DLG	equ	4400
ID_OP_TXTWBG	equ	4401
ID_OP_BTTWBG	equ	4402
ID_OP_TXITBG	equ	4403
ID_OP_BTITBG	equ	4404
ID_OP_TXTSTX	equ	4405
ID_OP_BTTSTX	equ	4406
ID_OP_TXTBBG	equ	4407
ID_OP_BTTBBG	equ	4408


; Help Dialog.

ID_HP_DLG	equ	4900
ID_HP_EDIT	equ	4901




; Global data ---------------------------------------------------------------


externdef	G_hTui:dword		; handle, Tui
externdef	G_ScrnWid:dword		; Screen width in pixels
externdef	G_ScrnHgt:dword		; Screen height in pixels
externdef	G_IsNT:dword		; flag, WinNT type OS present
externdef	G_GdipOk:dword		; flag, true if GDI+ loaded ok
externdef	G_APIErrCode:dword	; API function error code
externdef	G_UDBhEvent:dword	; handle, UDB Event object
externdef	G_UDBhMutex:dword	; handle, UDB Mutex

externdef	G_UDBStatus:dword	; UpdateDB status:
UDBCntFS	equ	00000001b	;	Counting files on all drives
UDBWaitFS	equ	00000010b	;	Waiting for filesystem change
UDBCreate	equ	00000100b	;	Creating database
UDBWaitUP	equ	00001000b	; 	Waiting for update permission
UDBUpd		equ	00010000b	;	Updating database
UDB1stDone	equ	00100000b	; 	Initial update finished
UDBDbStale	equ	10000000b	; 	Database is stale - change
					;	detected but update not done

externdef	G_szNewLine:byte	; CR,LF,0
externdef	G_szTAB:byte 		; TAB,0
externdef	G_szSpc:byte		; ' ',0

externdef	G_Tui_Exit:near		; abort label (near = far in Win32)


; GDI+ function name and invoke prototyping macro.

GdiplusStartup			DDPROTO	(pGdiplusStartup,3)
GdiplusShutdown			DDPROTO (pGdiplusShutdown,1)
GdipGetInterpolationMode	DDPROTO (pGdipGetInterpolationMode,2)
GdipSetInterpolationMode	DDPROTO (pGdipSetInterpolationMode,2)
GdipCreateBitmapFromScan0	DDPROTO (pGdipCreateBitmapFromScan0,6)
GdipCreateFromHDC		DDPROTO (pGdipCreateFromHDC,2)
GdipDeleteGraphics		DDPROTO (pGdipDeleteGraphics,1)
GdipDisposeImage		DDPROTO (pGdipDisposeImage,1)
GdipDrawImageRectI		DDPROTO (pGdipDrawImageRectI,6)
GdipGetImageGraphicsContext	DDPROTO (pGdipGetImageGraphicsContext,2)
GdipGraphicsClear		DDPROTO (pGdipGraphicsClear,2)
GdipLoadImageFromFile		DDPROTO (pGdipLoadImageFromFile,2)
GdipGetImageWidth		DDPROTO	(pGdipGetImageWidth,2)
GdipGetImageHeight		DDPROTO	(pGdipGetImageHeight,2)

externdef	pGdiplusStartup:dword
externdef	pGdiplusShutdown:dword
externdef	pGdipGetInterpolationMode:dword
externdef	pGdipSetInterpolationMode:dword
externdef	pGdipCreateBitmapFromScan0:dword
externdef	pGdipCreateFromHDC:dword
externdef	pGdipDeleteGraphics:dword
externdef	pGdipDisposeImage:dword
externdef	pGdipDrawImageRectI:dword
externdef	pGdipGetImageGraphicsContext:dword
externdef	pGdipGraphicsClear:dword
externdef	pGdipLoadImageFromFile:dword
externdef	pGdipGetImageWidth:dword
externdef	pGdipGetImageHeight:dword




; GDI+ structures and definitions -------------------------------------------
;
; Tui requires only 3 definitions.


; 1. GdiplusStartupInput
;
; This structure is used in tui.asm, function GdipStart.

GdiplusStartupInput		struct
  GdiplusVersion		dword	?	; Must be 1
  DebugEventCallback		dword	?	; Ignored on free builds
  SuppressBackgroundThread	dword	?
  SuppressExternalCodecs	dword	?
GdiplusStartupInput		ends

; Set SuppressBackgroundThread false unless you're prepared to call the
; hook/unhook functions properly.
;
; Set SuppressExternalCodecs false unless you want GDI+ to only use its
; internal image codecs.


; 2. Pixel Format
;
; PixelFormat32bppARGB is used in time.asm, TimeWndHdlr.

; In-memory pixel data formats:
;
; bits 00-07	format index
; bits 08-15	pixel size (in bits)
; bits 16-23	flags (below)
; bits 24-31	reserved

PixelFormatIndexed		= 00010000h	; Indexes into a palette
PixelFormatGDI			= 00020000h	; Is a GDI-supported format
PixelFormatAlpha		= 00040000h	; Has an alpha component
PixelFormatPAlpha		= 00080000h	; Uses pre-multiplied alpha
PixelFormatExtended		= 00100000h	; Uses extended color (16b/ch)
PixelFormatCanonical		= 00200000h	; ?
PixelFormatUndefined		= 0
PixelFormatDontCare		= 0

;				 Index	pixel_size	flags

PixelFormat1bppIndexed		=  1 or ( 1 shl 8) or PixelFormatIndexed \
						   or PixelFormatGDI
PixelFormat4bppIndexed		=  2 or ( 4 shl 8) or PixelFormatIndexed \
						   or PixelFormatGDI
PixelFormat8bppIndexed		=  3 or ( 8 shl 8) or PixelFormatIndexed \
						   or PixelFormatGDI
PixelFormat16bppGrayScale	=  4 or (16 shl 8) or PixelFormatExtended
PixelFormat16bppRGB555		=  5 or (16 shl 8) or PixelFormatGDI
PixelFormat16bppRGB565		=  6 or (16 shl 8) or PixelFormatGDI
PixelFormat16bppARGB1555	=  7 or (16 shl 8) or PixelFormatGDI \
						   or PixelFormatAlpha
PixelFormat24bppRGB		=  8 or (24 shl 8) or PixelFormatGDI
PixelFormat32bppRGB		=  9 or (32 shl 8) or PixelFormatGDI
PixelFormat32bppARGB		= 10 or (32 shl 8) or PixelFormatGDI \
						   or PixelFormatAlpha \
						   or PixelFormatCanonical
PixelFormat32bppPARGB		= 11 or (32 shl 8) or PixelFormatGDI \
						   or PixelFormatAlpha \
						   or PixelFormatPAlpha
PixelFormat48bppRGB		= 12 or (48 shl 8) or PixelFormatExtended
PixelFormat64bppARGB		= 13 or (64 shl 8) or PixelFormatAlpha \
						   or PixelFormatExtended \
						   or PixelFormatCanonical
PixelFormat64bppPARGB		= 14 or (64 shl 8) or PixelFormatAlpha \
						   or PixelFormatPAlpha \
						   or PixelFormatExtended
PixelFormat24bppBGR		= 15 or (24 shl 8) or PixelFormatGDI
PixelFormatMax			= 16


; 3. Quality mode
;
; InterpolationModeNearestNeighbor is used in TimeWndHdlr.

QualityModeInvalid			= -1
QualityModeDefault			= 0
QualityModeLow				= 1	; Best performance
QualityModeHigh				= 2	; Best rendering quality

InterpolationModeInvalid		= QualityModeInvalid
InterpolationModeDefault		= QualityModeDefault
InterpolationModeLowQuality		= QualityModeLow
InterpolationModeHighQuality		= QualityModeHigh
InterpolationModeBilinear		= 3
InterpolationModeBicubic		= 4
InterpolationModeNearestNeighbor	= 5
InterpolationModeHighQualityBilinear	= 6
InterpolationModeHighQualityBicubic	= 7




; Plugin specification ------------------------------------------------------
;
; This structure is required to be present and initialised in every plugin.
;
; NB An exact copy is held in \plugin\plugin.inc for the use of plugins.
;    If changes are made here then make there as well.


PLGSpec		struct
  pPLG		dd	?		; address of plugin handler
  Sfx1		db	4 dup (?)	; 4 char file suffix
  Sfx2		db	4 dup (?)	; possible additional suffix
  szName	db	16 dup (?)	; 15 char plugin name + terminator
PLGSpec		ends




;============================================================================
